from CLASSES import NxOnyxApi
from LIBS_CLI.parameter_selector import paramter_selector


def transco_id(comp,id_src,onyx_src : NxOnyxApi,onyx_trg : NxOnyxApi,paramters):


    dl=paramter_selector(paramters)



    dict={"SQL_SCRIPT":{},"CONNECTION":{}}
    for obj in comp.componants:

        if "osqlScript" in obj and 1 in dl:
            data = onyx_src.getSqlScript(obj.content["osqlScript"]["id"])
            get_sql_scripts = onyx_trg.getSqlScriptsByProject(id)

            for element in get_sql_scripts["items"]:
                if element["osqlScript"]["name"] == data["name"]:
                    trg_id = element["osqlScript"]["id"]

                    dict["SQL_SCRIPT"][obj["id"]]=trg_id

        if "oConnection" in obj and 2 in dl:
            data = onyx_src.getConnection(id_src)
            get_connection = onyx_trg.getConnectionsByProject(id)

            for element in get_connection["items"]:
                if element["oConnection"]["name"] == data["name"]:
                    trg_id = element["oConnection"]["id"]

                    dict["CONNECTION"][obj["id"]] = trg_id
    return dict


import json

class NxComponantAttributes:
    def __init__(self, json_object: str, comparable_attributes: []):
        self.json_object = json_object
        self.comparable_attributes = comparable_attributes




from CLASSES.NxTenant import NxTenant
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects
from LIBS_CLI.cli_get_organisation_unit import *


def deploy_oConnections(onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi, tenant_src: NxTenant,
                      tenant_trg: NxTenant, source_id, target_id, script_type,name):

    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type,name=name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type,name=name)
    comp = compare_onyx_projects(project_src, project_tnt)

    response=None
    for obj in comp.componants:
        response =obj.content["trg_id"]
        if "oConnection" in obj.content and obj.action_required == 'C' or obj.action_required == 'U':
            data = obj.content["oConnection"]
            ou_name = cli_get_organisation_unit_name(onyx_src, data["organizationUnitId"])
            ou_id = cli_get_organisation_unit_id(onyx_trg, ou_name)
            response=onyx_trg.createConnection(id=obj.content["trg_id"],connectionType=data["connectionType"],  isWritable=data["isWritable"],
                                               name=data["name"], organizationUnitId=ou_id, shortName=data["shortName"],documentation=data["documentation"])
            print("Please finish connection creation in Onyx in your target environement")

    return response



from LIBS_CLI.update_envconf import update_envconf
from CLASSES.EnvConfElement import EnvConfElement
from CLASSES.NxTenant import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
import os
from rich.console import Console
#new corrections
def action_1(tenant=None, tenantName=None, tenantDomain=None, tenantUsername=None, tenantPassword=None, tenantId=None):

    if tenant is not None:
        tenant=tenant.upper()
    if tenantName is not None:
        tenantName=tenantName.upper()

    dict= {"TENANT":tenant, "TENANTDOMAIN":tenantDomain,"TENANTID":tenantId,"TENANTNAME":tenantName,"TENANTUSERNAME":tenantUsername,"TENANTPASSWORD":tenantPassword}
    for key in dict:
        if dict[key]==None:
            if key =="TENANT":
                dict[key]=str(input("Enter "+ key +" : ")).upper()
            else:
                dict[key] = str(input("Enter " + key + " : "))

    try:
        tenant_dict = NxTenant(domain=dict["TENANTDOMAIN"], tenant_id=dict["TENANTID"],
                            tenant_name=dict["TENANTNAME"], username=dict["TENANTUSERNAME"],
                            password=dict["TENANTPASSWORD"])

        tenant_dict = NxOnyxApi(tenant_dict.domain, tenant_dict.username, tenant_dict.password, tenant_dict.tenant_id)
        print("Connection to source", dict["TENANTNAME"], "successful.")

        #conf_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../env.conf"))
        conf_file_path = os.path.join(os.getcwd(), "env.conf")

        if not os.path.exists(conf_file_path) or os.getenv(dict["TENANT"] + "_TENANTNAME") is None:
            with open(conf_file_path, 'a+') as file:
                file.write('\n#' + dict["TENANT"].upper() + '\n')
                for Element in dict:
                    if Element != "TENANT":
                        file.write(dict["TENANT"] + "_" + str(Element) + '=' + str(dict[Element]) + '\n')
            print("New env.conf file created in the src folder.")
        else:
            for Element in dict:
                if Element != "tenant":
                    update_envconf(EnvConfElement(filename=conf_file_path, key=dict["TENANT"] + "_" + Element,
                                                value=dict[Element]))
            print("Tenant information updated in env.conf.")

    except Exception as e:
        console = Console()
        console.print("Connection to tenant  impossible:" + str(e), style="red")


import click
import warnings
from LIBS_CLI.action_5 import action_5
from LIBS_CLI.action_1 import action_1
from LIBS_CLI.check_project_existence import check_project_existence
from LIBS_CLI.cli_tenant_validation import cli_tenant_validation
from LIBS_CLI.get_tenantname import get_tenantname
from LIBS_CLI.action_2 import action_2
from LIBS_CLI.action_3 import action_3
from LIBS_CLI.action_4 import action_4
from LIBS_CLI.get_envconf import get_envconf
from LIBS_CLI import load_envconf
from LIBS_CLI.exist_envconf import exist_envconf
import urllib3
from urllib3.exceptions import InsecureRequestWarning

# Suppress InsecureRequestWarning
urllib3.disable_warnings(InsecureRequestWarning)

# Suppress other warnings
warnings.filterwarnings('ignore')

@click.group()
def onyxpm():

    dotenv_file = get_envconf()
    env = load_envconf(dotenv_file)

@onyxpm.group()
def tenant():
    pass

@onyxpm.group()
def project():
    pass

@tenant.command(help="Configure parameters - tenant, tenantname, tenantdomain, tenantusername, tenantpassword, tenantid")
@click.option('-t', '--tenant', help='Set the Tenant parameters. Example: "prod"')
@click.option('-tn', '--tenantname', help='Set the Tenant Name. Example: "MyTenant"')
@click.option('-td', '--tenantdomain', help='Set the Tenant Domain. Example: "example.com"')
@click.option('-tu', '--tenantusername', help='Set the Tenant Username. Example: "user"')
@click.option('-tp', '--tenantpassword', help='Set the Tenant Password. Example: "password"')
@click.option('-ti', '--tenantid', help='Set the Tenant ID. Example: "123456"')
def set(tenant, tenantname, tenantdomain, tenantusername, tenantpassword, tenantid):
    """
    usage: onyxpm.py tenant set [OPTIONS]

    configure parameters - tenant, tenantname, tenantdomain, tenantusername, tenantpassword, tenantid

    Options:
    -t, --tenant TEXT                            Set the Tenant Parameters. Example: "prod"
    -tn, --tenantname TEXT                       Set the Tenant Name. Example: "MyTenant"
    -td, --tenantdomain TEXT                     Set the Tenant Domain. Example: "example.com"
    -tu, --tenantusername TEXT                   Set the Tenant Username. Example: "user"
    -tp, --tenantpassword TEXT                   Set the Tenant Password. Example: "password"
    -ti, --tenantid TEXT                         Set the Tenant ID. Example: "123456"
    --help                                       Show this message and exit
    """
    action_1(tenant, tenantname, tenantdomain, tenantusername, tenantpassword, tenantid)

@tenant.command(help="List all available tenants.")
def list():
    if exist_envconf() != False:
        get_tenantname()

@project.command(help='List of projects in the tenant')
@click.option('-t', '--tenant', help='List of projects in the tenant', required=True)
def list(tenant):
    if exist_envconf() != False:
        action_5(tenant)
    if not tenant:
        click.echo("Please specify a valid tenant name.")


@tenant.command(help='Create a directory and store all project components')
@click.option('-t', '--tenant', help='Tenant name', required=True)
@click.option('-f', '--folder', help='Folder path', required=True)
def dump(tenant, folder):
    if exist_envconf() != False:
        click.echo(f'Dumping data for tenant {tenant} to folder {folder}')
        action_4(tenant, folder)
    if not tenant:
        click.echo("Please specify a valid tenant name.")

@project.command(help='Compare the desired projects in source and target tenants')
@click.option('-ts', '--source_tenant', help='Source tenant name', required=True)
@click.option('-tt', '--target_tenant', help='Target tenant name', required=True)
@click.option('-p', '--project', help='Project name', required=True)
def compare(source_tenant, target_tenant, project):
    if exist_envconf() != False:
        # Call the cli_tenant_validation function
        if not cli_tenant_validation(source_tenant, target_tenant):
            return

        # Check if the project exists
        project_exists = check_project_existence(source_tenant, target_tenant, project)
        if project_exists:
            click.echo(f'Comparing projects for source tenant {source_tenant} to target tenant {target_tenant} for project {project}')
            action_2(source_tenant, target_tenant, project)

@project.command(help='Choose to update your target tenant with desired project or a specfic object in a project from the source tenant.')
@click.option('-i', '--script_type', help='Script type')
@click.option('-o', '--object_name', help='Object name', default='ALL')
@click.option('-p', '--project', help='Project name', required=True) #possibilité de le rendre non obligatoire (j'ai envie de le faire)
@click.option('-ts', '--source_tenant', help='Source tenant name', required=True)
@click.option('-tt', '--target_tenant', help='Target tenant name', required=True)
def deploy(script_type, project, source_tenant, target_tenant, object_name):
    if exist_envconf() != False:
        script_type=script_type.upper()
        # Call the cli_tenant_validation function
        if not cli_tenant_validation(source_tenant, target_tenant):
            return
        project_exists = check_project_existence(source_tenant, target_tenant, project)
        if project_exists:
            click.echo(f'Deploying {script_type} script {object_name} for project {project} from {source_tenant} to {target_tenant}')
            action_3(script_type, object_name, project, source_tenant, target_tenant)

if __name__ == '__main__':
    onyxpm()




import json


class NxProject:
    def __init__(self, id: str, name:str, content:json, componants:[]):
        self.id = id
        self.name = name
        self.content = content
        self.componants = componants



class NxTenant:
    def __init__(self, domain, tenant_id, tenant_name, username, password):
        self.domain = domain
        self.username = username
        self.password = password
        self.tenant_id = tenant_id
        self.tenant_name = tenant_name


import requests
import json

class NxOnyxApi:
    def __init__(self, domain, username, password, tenantId):
        self.__domain = domain
        self.__username = username
        self.__password = password
        self.__tenantId = tenantId
        self.__token = self.getToken()

    def getToken(self):
        url = f"{self.__domain}/api/TokenAuth/Authenticate"

        payload = json.dumps({
            "userNameOrEmailAddress": self.__username,
            "password": self.__password
        })
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        }
        try:
            response = requests.post(url, headers=headers, data=payload, verify=False)
            if response.status_code == 200:
                token = response.json().get('result').get('accessToken')
                # print("Accessed bearer token and connection successful : {}".format(token))
                return token
            else:
                response = requests.post(url, headers=headers, data=payload)
                raise Exception(f"Connection failed with status code: {response.status_code}, {response.json()}")

        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred in getToken:\n" + str(e))

    def startWorkflow(self, workflowId):
        url = f"{self.__domain}/api/services/app/OWorkflows/Start"

        payload = json.dumps({
            "id": workflowId
        })
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'application/json',
            'Authorization': f'Bearer {self.__token}',
            'Content-Type': 'application/json'
        }

        try:
            response = requests.post(url, headers=headers, data=payload)
            response_json = response.json()

            if response.status_code == 200:
                scriptId = response_json.get('result')
                print("Workflow successfully created")
                return scriptId
            else:
                check_if_already_running = 'Une instance de la tâche est déjà en cours' in response_json
                if not check_if_already_running:
                    raise Exception(f"StartWorkFlow failed with status code: {response.status_code}, {response_json}")

        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred in StartWorkFow:\n" + str(e))

    def registerSchedule(self, scheduleId):
        url = f"{self.__domain}/api/services/app/OSchedules/Register?id=" + scheduleId
        payload = json.dumps({
            "id": scheduleId
        })
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'application/json',
            'Authorization': f'Bearer {self.__token}',
            'Content-Type': 'application/json'
        }
        try:
            response = requests.post(url, headers=headers, data={})
            response_json = response.json()

            if response.status_code == 200:
                scriptId = response_json.get('result')
                print("Schedule successfully activated")
                return scriptId
            else:
                raise Exception(f"Activation failed with status code: {response.status_code}, {response_json}")

        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred in activation :\n" + str(e))

    def unregisterSchedule(self, scheduleId):
        url = f"{self.__domain}/api/services/app/OSchedules/Unregister?id=" + scheduleId
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'application/json',
            'Authorization': f'Bearer {self.__token}',
            'Content-Type': 'application/json'
        }

        try:
            response = requests.post(url, headers=headers, data={})
            response_json = response.json()

            if response.status_code == 200:
                scriptId = response_json.get('result')
                print("Schedule successfully desactivated")
                return scriptId
            else:
                raise Exception(f"Desactivation failed with status code: {response.status_code}, {response_json}")

        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred in Desactivation :\n" + str(e))

    def getTree(self):
        url = f"{self.__domain}/api/services/app/OProjects/GetTree"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                tree = response_json.get('result')
                return tree
            else:
                raise Exception(f"GetTree failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getOrganisationUnits(self):
        url = f"{self.__domain}/api/services/app/OrganizationUnit/GetOrganizationUnits"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }

        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                tree = response_json.get('result')
                return tree
            else:
                raise Exception(
                    f"GetOrganisationUnits failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getProject(self, id):
        url = f"{self.__domain}/api/services/app/OProjects/GetOProjectForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                tree = response_json.get('result')
                return tree
            else:
                raise Exception(f"GetProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createProject(self, name, organizationUnitId: int):
        url = f"{self.__domain}/api/services/app/OProjects/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'name': name,
            'organizationUnitId': organizationUnitId
        }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                tree = response_json.get('result')
                return tree
            else:
                raise Exception(f"GetProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getConnection(self, id):
        url = f"{self.__domain}/api/services/app/OConnections/GetOConnectionForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                tree = response_json.get('result')
                return tree
            else:
                raise Exception(f"GetConnection failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getConnectionsByProject(self, id):
        url = f"{self.__domain}/api/services/app/OConnections/GetConnectionsByProject?ProjectId=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(
                    f"GetConnectionsByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getFileProvidersByProject(self, id):
        url = f"{self.__domain}/api/services/app/OFileProviders/GetFileProvidersByProject?ProjectId=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(
                    f"GetFileProvidersByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getFileProvider(self, id):
        url = f"{self.__domain}/api/services/app/OFileProviders/GetOFileProviderForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"GetFileProvider failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createFileProvider(self, connectionString, containerName, name, organizationUnitId, type, id=None):
        url = f"{self.__domain}/api/services/app/OFileProviders/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'connectionString': connectionString,
            'containerName': containerName,
            'name': name,
            'organizationUnitId': organizationUnitId,
            'type': type
        }
        if not id == None:
            payload = {
                'connectionString': connectionString,
                'containerName': containerName,
                'name': name,
                'organizationUnitId': organizationUnitId,
                'type': type,
                'id': id
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateFileProvider failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getConnections(self):
        url = f"{self.__domain}/api/services/app/OConnections/GetAll"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"GetConnections failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createConnection(self, connectionType: int, isWritable: bool, name: str, organizationUnitId: int,
                         shortName: str, documentation, id=None):
        url = f"{self.__domain}/api/services/app/OConnections/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'connectionType': connectionType,
            'isWritable': isWritable,
            'name': name,
            'organizationUnitId': organizationUnitId,
            'shortName': shortName,
            'documentation': documentation,
        }
        if not id == None:
            payload = {
                'connectionType': connectionType,
                'isWritable': isWritable,
                'name': name,
                'organizationUnitId': organizationUnitId,
                'shortName': shortName,
                'documentation':documentation,
                'id': id
            }

        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateConnection failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def updateConnectionString(self, id, ConnectionString):
        url = f"{self.__domain}/api/services/app/OConnections/UpdateCS"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'id': id,
            'ConnectionString': ConnectionString
        }
        try:
            print(payload)
            response = requests.put(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(
                    f"UpdateConnectionString failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getVariablesByProject(self, id):
        url = f"{self.__domain}/api/services/app/OVariable/GetAll?Filter=&NameFilter=&CodeFilter=&OProjectIdFilter=" + id + "&SkipCount=0&MaxResultCount=100"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(
                    f"GetVariablesByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getVariable(self, id):
        url = f"{self.__domain}/api/services/app/OVariable/GetOVariableForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"GetVariable failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createVariable(self, code, name, description, oConnectionId, oProjectId, query, value,oVariableType, tenantId, id=None):
        url = f"{self.__domain}/api/services/app/OVariable/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'name': name,
            'code': code,
            'description': description,
            'value': value,
            'oConnectionId': oConnectionId,
            'oProjectId': oProjectId,
            'query': query,
            'oVariableType': oVariableType,
            'tenantId': tenantId
        }
        if not id == None:
            payload = {
                'id': id,
                'name': name,
                'code': code,
                'description': description,
                'value': value,
                'oConnectionId': oConnectionId,
                'oProjectId': oProjectId,
                'query': query,
                'oVariableType': oVariableType,
                'tenantId': tenantId
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateVariable failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def updateVariable(self, id, code, name, oConnectionId, oProjectId, query, value, oVariableType):
        url = f"{self.__domain}/api/services/app/OVariable/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'tenantId': self.__tenantId,
            'id': id,
            'code': code,
            'name': name,
            'oConnectionId': oConnectionId,
            'oProjectId': oProjectId,
            'query': query,
            'value': value,
            'oVariableType': oVariableType
        }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateFileProvider failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getSqlScriptsByProject(self, id):
        url = f"{self.__domain}/api/services/app/OSQLScripts/GetAll?NameFilter=&OProjectNameFilter=" + id + "&OConnectionNameFilter=&Sorting=&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(
                    f"GetSqlScriptsByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getSqlScript(self, id):
        url = f"{self.__domain}/api/services/app/OSQLScripts/GetOSQLScriptForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetSqlScript failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createSqlScript(self, name, oConnectionId, oProjectId, commandTimeout, query, documentation, id=None):
        url = f"{self.__domain}/api/services/app/OSQLScripts/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'commandTimeout': commandTimeout,
            'name': name,
            'oConnectionId': oConnectionId,
            'oProjectId': oProjectId,
            'query': query,
            'documentation': documentation
        }
        if not id == None:
            payload = {
                'id': id,
                'commandTimeout': commandTimeout,
                'name': name,
                'oConnectionId': oConnectionId,
                'oProjectId': oProjectId,
                'query': query,
                'documentation': documentation
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateSqlScript failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createShellScript(self, name, oProjectId, type, script, documentation, packages, id=None):
        url = f"{self.__domain}/api/services/app/OShellScripts/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'type': type,
            'name': name,
            'oProjectId': oProjectId,
            'script': script,
            'documentation': documentation,
            'packages': packages
        }
        if not id == None:
            payload = {
                'id': id,
                'type': type,
                'name': name,
                'oProjectId': oProjectId,
                'script': script,
                'documentation': documentation,
                'packages': packages
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateShellScript failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createNotification(self, name, oProjectId, recipients, subject, body, tenant, type, id=None):
        url = f"{self.__domain}/api/services/app/ONotifications/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'name': name,
            'oProjectId': oProjectId,
            'recipients': recipients,
            'subject': subject,
            'tenantId': tenant,
            'type' : type,
            'body': body
        }
        if not id == None:
            payload = {
                'name': name,
                'recipients': recipients,
                'subject': subject,
                'body': body,
                'oProjectId': oProjectId,
                'tenantId': tenant,
                'type' : type,
                'id': id
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"createNotification failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createForm(self, displayedName, oProjectId, oConnectionId, technicalName, isActive, documentation, id=None):
        url = f"{self.__domain}/api/services/app/OForms/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'displayedName': displayedName,
            'oProjectId': oProjectId,
            'oConnectionId': oConnectionId,
            'technicalName': technicalName,
            'isActive': isActive,
            'documentation': documentation
        }
        if not id == None:
            payload = {
                'displayedName': displayedName,
                'oProjectId': oProjectId,
                'oConnectionId': oConnectionId,
                'technicalName': technicalName,
                'isActive': isActive,
                'documentation': documentation,
                'id': id
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"createForm failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createFormColumn(self, characterMaximumLength, displayOrder, displayedName, dropDownQuery, editorType,
                         isDisplayed, isDropDown, isNullable, numericPrecision, technicalName, numericScale, oFormId,
                         type, id=None):
        url = f"{self.__domain}/api/services/app/OFormColumns/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'characterMaximumLength': characterMaximumLength,
            'displayOrder': displayOrder,
            'displayedName': displayedName,
            'technicalName': technicalName,
            'dropDownQuery': dropDownQuery,
            'editorType': editorType,
            'isDisplayed': isDisplayed,
            'isDropDown': isDropDown,
            'isNullable': isNullable,
            'numericPrecision': numericPrecision,
            'numericScale': numericScale,
            'oFormId': oFormId,
            'type': type
        }
        if not id == None:
            payload = {
                'characterMaximumLength': characterMaximumLength,
                'displayOrder': displayOrder,
                'displayedName': displayedName,
                'technicalName': technicalName,
                'dropDownQuery': dropDownQuery,
                'editorType': editorType,
                'isDisplayed': isDisplayed,
                'isDropDown': isDropDown,
                'isNullable': isNullable,
                'numericPrecision': numericPrecision,
                'numericScale': numericScale,
                'oFormId': oFormId,
                'id': id
            }

        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"createForm failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getShellScriptsByProject(self, id):
        url = f"{self.__domain}/api/services/app/OShellScripts/GetAll?NameFilter=&OProjectIdFilter=" + id + "&Sorting=&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(
                    f"GetShellScriptsByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getShellScript(self, id):
        url = f"{self.__domain}/api/services/app/OShellScripts/GetOShellScriptForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetShellScript failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getFormsByProject(self, id):
        url = f"{self.__domain}/api/services/app/OForms/GetAll?OProjectIdFilter=" + id + "&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetFormsByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getForm(self, id):
        url = f"{self.__domain}/api/services/app/OForms/GetOFormForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetForm failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getFormColumns(self, id):
        url = f"{self.__domain}/api/services/app/OFormColumns/GetAll?Filter=&TechnicalNameFilter=&DisplayedNameFilter=&DropDownQueryFilter=&OFormTechnicalNameFilter=" + id + "&Sorting=&SkipCount=0"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetFormColumns failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWidgetsByProject(self, id):
        url = f"{self.__domain}/api/services/app/OWidgets/GetAll?OProjectId=" + id + "&WithFilters=false&WithRowButtons=false&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetWidgetsByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWidget(self, id):
        url = f"{self.__domain}/api/services/app/OWidgets/GetOWidgetForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetWidget failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWidgetFilters(self, id):
        url = f"{self.__domain}/api/services/app/OWidgets/GetAllFilters?OWidgetId=" + id + "&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetWidgetFilters failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWidgetRowButtons(self, id):
        url = f"{self.__domain}/api/services/app/OWidgets/GetAllRowButtons?OWidgetId=" + id + "&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetWidgetRowButtons failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getReportsByProject(self, id):
        url = f"{self.__domain}/api/services/app/OReports/GetAll?OProjectId=" + id + "&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetReportsByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getReportForEdit(self, id):
        url = f"{self.__domain}/api/services/app/OReports/GetOReportForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetReportForEdit failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getReport(self, id):
        url = f"{self.__domain}/api/services/app/OReports/GetOReportForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GeReport failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getSchedulesByProject(self, id):
        url = f"{self.__domain}/api/services/app/OSchedules/GetAll?nameFilter=&ObjectIdFilter=&OProjectNameFilter=" + id + "&Sorting=&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(
                    f"GetSchedulesByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getSchedule(self, id):
        url = f"{self.__domain}/api/services/app/OSchedules/GetOScheduleForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            "Id": id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetSchedule failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getNotificationsByProject(self, id):
        url = f"{self.__domain}/api/services/app/ONotifications/GetAll?NameFilter=&OProjectIdFilter=" + id + "&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(
                    f"GetNotificationsByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getNotification(self, id):
        url = f"{self.__domain}/api/services/app/ONotifications/GetONotificationForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            "Id": id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetNotification failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWorkflowsByProject(self, id):
        url = f"{self.__domain}/api/services/app/OWorkflows/GetAll?NameFilter=&OProjectIdFilter=" + id + "&Sorting=&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(
                    f"GetWorkflowsByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWorkflow(self, id):
        url = f"{self.__domain}/api/services/app/OWorkflows/GetOWorkflowForEdit?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            "Id": id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetWorkflow failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWorkflowSteps(self, id):
        url = f"{self.__domain}/api/services/app/OWorkflows/GetAllSteps?OWorkflowIdFilter=" + id + "&Sorting=&SkipCount=0&MaxResultCount=1000"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            "Id": id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetWorkflowSteps failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWorkflowStepForView(self, id):
        url = f"{self.__domain}/api/services/app/OWorkflows/GetOWorkflowStepForView?id" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            "Id": id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetWorkflowSteps failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))



    def getPipelinesByProject(self, id):
        url = f"{self.__domain}/api/services/app/OProjects/GetFlows?input=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.get(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:

                return response_json.get('result')
            else:
                raise Exception(
                    f"GetPipelinesByProject failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getPipeline(self, id):
        url = f"{self.__domain}/api/services/app/OProjects/GetFlow?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            "Id": id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetPipeline failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getPipelineColumnsIncluded(self, id):
        url = f"{self.__domain}/api/services/app/OProjects/GetFlowColumns?input=" + id + "&included=true"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            "input": id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetPipelineColumns failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getPipelineColumnsExcluded(self, id):
        url = f"{self.__domain}/api/services/app/OProjects/GetFlowColumns?input=" + id + "&included=false"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            "input": id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetPipelineColumns failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createPipeline(self, id, oProjectId, SourceConnectionId, SourceConnectionType, DestinationConnectionId,
                       DestinationConnectionType, Name, SourceTable, DestinationTable, QueryFilter, NbColumns,
                       ActionIfTableExists, ActionIfTableNotExists, DesactivateIndexes, CreatePrimaryKey, CreateIndexes,
                       NumberOfSchedules):
        url = f"{self.__domain}/api/services/app/OProjects/AddFlow"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        if id == None:
            payload = {
                'oProjectId': oProjectId,
                'SourceConnectionId': SourceConnectionId,
                'SourceConnectionType': SourceConnectionType,
                'DestinationConnectionId': DestinationConnectionId,
                'DestinationConnectionType': DestinationConnectionType,
                'Name': Name,
                'SourceTable': SourceTable,
                'DestinationTable': DestinationTable,
                'QueryFilter': QueryFilter,
                'NbColumns': NbColumns,
                'ActionIfTableExists': ActionIfTableExists,
                'ActionIfTableNotExists': ActionIfTableNotExists,
                'DesactivateIndexes': DesactivateIndexes,
                'CreatePrimaryKey': CreatePrimaryKey,
                'CreateIndexes': CreateIndexes,
                'NumberOfSchedules': NumberOfSchedules
            }
            try:
                response = requests.post(url, headers=headers, json=payload)
                response_json = response.json()
                if response.status_code == 200:
                    cnx = response_json.get('result')
                    return cnx
                else:
                    raise Exception(f"CreatePipeline failed with status code: {response.status_code}, {response_json}")
            except requests.exceptions.RequestException as e:
                raise ValueError("An error occurred :\n" + str(e))
        if not id == None:
            url = f"{self.__domain}/api/services/app/OProjects/UpdateFlow"
            payload = {
                'oProjectId': oProjectId,
                'SourceConnectionId': SourceConnectionId,
                'SourceConnectionType': SourceConnectionType,
                'DestinationConnectionId': DestinationConnectionId,
                'DestinationConnectionType': DestinationConnectionType,
                'Name': Name,
                'SourceTable': SourceTable,
                'DestinationTable': DestinationTable,
                'QueryFilter': QueryFilter,
                'NbColumns': NbColumns,
                'ActionIfTableExists': ActionIfTableExists,
                'ActionIfTableNotExists': ActionIfTableNotExists,
                'DesactivateIndexes': DesactivateIndexes,
                'CreatePrimaryKey': CreatePrimaryKey,
                'CreateIndexes': CreateIndexes,
                'NumberOfSchedules': NumberOfSchedules,
                'id': id
            }
            try:
                response = requests.put(url, headers=headers, json=payload)
                response_json = response.json()
                if response.status_code == 200:
                    cnx = response_json.get('result')
                    return cnx
                else:
                    raise Exception(f"CreatePipeline failed with status code: {response.status_code}, {response_json}")
            except requests.exceptions.RequestException as e:
                raise ValueError("An error occurred :\n" + str(e))

    def createPipelineColumn(self, oFlowId, sourceColumnName, destinationColumnName, destinationColumnDataType,
                             sourceCharacterMaximumLength, sourceColumnNumericPrecision, sourceColumnNumericScale,
                             sourceColumnDatatype, ordinal, missingInSource,id):
        if id is None:
            url = f"{self.__domain}/api/services/app/OProjects/AddFlowColumn"
            headers = {
                'Abp.TenantId': self.__tenantId,
                'Accept': 'text/plain',
                'Authorization': f'Bearer {self.__token}'
            }
            payload = {
                "oFlowId": oFlowId,
                "sourceColumnName": sourceColumnName,
                "destinationColumnName": destinationColumnName,
                "destinationColumnDataType": destinationColumnDataType,
                "sourceCharacterMaximumLength": sourceCharacterMaximumLength,
                "sourceColumnNumericPrecision": sourceColumnNumericPrecision,
                "sourceColumnNumericScale": sourceColumnNumericScale,
                "sourceColumnDatatype": sourceColumnDatatype,
                "ordinal": ordinal,
                "missingInSource": missingInSource
            }
            try:
                response = requests.post(url, headers=headers, json=payload)
                response_json = response.json()
                if response.status_code == 200:
                    cnx = response_json.get('result')
                    return cnx
                else:
                    raise Exception(
                        f"CreatePipelineColumn failed with status code: {response.status_code}, {response_json}")
            except requests.exceptions.RequestException as e:
                raise ValueError("An error occurred :\n" + str(e))
        else:
            url=f"{self.__domain}/api/services/app/OProjects/UpdateFlowColumn"
            headers = {
                'Abp.TenantId': self.__tenantId,
                'Accept': 'text/plain',
                'Authorization': f'Bearer {self.__token}'
            }
            payload = {
                "oFlowId": oFlowId,
                "sourceColumnName": sourceColumnName,
                "destinationColumnName": destinationColumnName,
                "destinationColumnDataType": destinationColumnDataType,
                "sourceCharacterMaximumLength": sourceCharacterMaximumLength,
                "sourceColumnNumericPrecision": sourceColumnNumericPrecision,
                "sourceColumnNumericScale": sourceColumnNumericScale,
                "sourceColumnDatatype": sourceColumnDatatype,
                "ordinal": ordinal,
                "missingInSource": missingInSource,
                "id": id
            }
            try:
                response = requests.put(url, headers=headers, json=payload)
                response_json = response.json()
                if response.status_code == 200:
                    cnx = response_json.get('result')
                    return cnx
                else:
                    raise Exception(
                        f"CreatePipelineColumn failed with status code: {response.status_code}, {response_json}")
            except requests.exceptions.RequestException as e:
                raise ValueError("An error occurred :\n" + str(e))

    def deletePipelineColumn(self, id):
        url = f"{self.__domain}/api/services/app/OProjects/DeleteFlowColumn?input=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.delete(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(
                    f"DeletePipelineColumn failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createWorkflow(self, oProjectId, name, enableCrossProject, emailSentOnError, emailRecipients, id=None):
        url = f"{self.__domain}/api/services/app/OWorkflows/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'oProjectId': oProjectId,
            'name': name,
            'enableCrossProject': enableCrossProject,
            'emailSentOnError': emailSentOnError,
            'emailRecipients': emailRecipients
        }
        if not id == None:
            payload = {
                'oProjectId': oProjectId,
                'name': name,
                'enableCrossProject': enableCrossProject,
                'emailSentOnError': emailSentOnError,
                'emailRecipients': emailRecipients,
                'id': id
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateWorkflow failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createWorkflowStep(self, oWorkflowId, objectId, stepOrder, stopWorkflowOnError, isActive, jobType):
        url = f"{self.__domain}/api/services/app/OWorkflows/CreateOrEditStep"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'oWorkflowId': oWorkflowId,
            'objectId': objectId,
            'stepOrder': stepOrder,
            'stopWorkflowOnError': stopWorkflowOnError,
            'isActive': isActive,
            'jobType': jobType
        }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateWorkflowStep failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def deleteWorkflowStep(self, id):
        url = f"{self.__domain}/api/services/app/OWorkflows/DeleteStep?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.delete(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"DeleteWorkflowStep failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createWidget(self, oProjectId, name, description, query, configuration, oConnectionId, oWorkflowId, oFormId,
                     oFileProviderId, type,id=None):
        url = f"{self.__domain}/api/services/app/OWidgets/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'oProjectId': oProjectId,
            'name': name,
            'description': description,
            'query': query,
            'configuration': configuration,
            'oConnectionId': oConnectionId,
            'oWorkflowId': oWorkflowId,
            'oFormId': oFormId,
            'oFileProviderId': oFileProviderId,
            'type': type
        }
        if not id == None:
            payload = {
                'oProjectId': oProjectId,
                'name': name,
                'description': description,
                'query': query,
                'configuration': configuration,
                'oConnectionId': oConnectionId,
                'oWorkflowId': oWorkflowId,
                'oFormId': oFormId,
                'oFileProviderId': oFileProviderId,
                'type': type,
                'id': id
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateWidget failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def deleteWidgetFilter(self, id):
        url = f"{self.__domain}/api/services/app/OWidgets/DeleteFilter?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.delete(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"DeleteWidgetFilter failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def deleteWidgetRowButton(self, id):
        url = f"{self.__domain}/api/services/app/OWidgets/DeleteRowButton?Id=" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        try:
            response = requests.delete(url, headers=headers)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(
                    f"DeleteWidgetRowButton failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createWidgetFilter(self, name, description, oConnectionId, oWidgetId, query, type, id=None):
        url = f"{self.__domain}/api/services/app/OWidgets/CreateOrEditFilter"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'name': name,
            'description': description,
            'oConnectionId': oConnectionId,
            'oWidgetId': oWidgetId,
            'query': query,
            'type': type
        }

        if not id == None:
            payload = {
                'name': name,
                'description': description,
                'oConnectionId': oConnectionId,
                'oWidgetId': oWidgetId,
                'query': query,
                'type': type,
                'id':id
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateWidgetFilter failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createWidgetRowButton(self, actionType, configuration, description, name, oSqlScriptId, oWidgetId, oWorkflowId,id):
        url = f"{self.__domain}/api/services/app/OWidgets/CreateOrEditRowButton"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'actionType': actionType,
            'configuration': configuration,
            'description': description,
            'name': name,
            'oSqlScriptId': oSqlScriptId,
            'oWidgetId': oWidgetId,
            'oWorkflowId': oWorkflowId,
            "id":id
        }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(
                    f"CreateWidgetRowButton failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def getWidgetForView(self, id):
        url = f"{self.__domain}/api/services/app/OWidgets/GetOWidgetForView?id" + id
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = json.dumps({
            'id': id
        })
        try:
            response = requests.get(url, headers=headers, data=payload)
            response_json = response.json()
            if response.status_code == 200:
                return response_json.get('result')
            else:
                raise Exception(f"GetWidgetForView failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))

    def createReport(self, name, oProjectId, documentation, configuration, id=None):
        url = f"{self.__domain}/api/services/app/OReports/CreateOrEdit"
        headers = {
            'Abp.TenantId': self.__tenantId,
            'Accept': 'text/plain',
            'Authorization': f'Bearer {self.__token}'
        }
        payload = {
            'name': name,
            'oProjectId': oProjectId,
            'configuration': configuration,
            'documentation': documentation
        }
        if not id == None:
            payload = {
                'name': name,
                'oProjectId': oProjectId,
                'configuration': configuration,
                'documentation': documentation,
                'id': id
            }
        try:
            response = requests.post(url, headers=headers, json=payload)
            response_json = response.json()
            if response.status_code == 200:
                cnx = response_json.get('result')
                return cnx
            else:
                raise Exception(f"CreateReport failed with status code: {response.status_code}, {response_json}")
        except requests.exceptions.RequestException as e:
            raise ValueError("An error occurred :\n" + str(e))



from CLASSES import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects
from LIBS_CLI.cli_add_to_workflow import add_to_workflow_shellscript, add_to_workflow_sqlscript, add_to_workflow_notification

def deploy_oWorkFlows( onyx_src : NxOnyxApi, onyx_trg : NxOnyxApi, tenant_src : NxTenant, tenant_trg : NxTenant, source_id, target_id, script_type,object_name):
    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type,name=object_name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type,name=object_name)
    comp = compare_onyx_projects(project_src, project_tnt)

    new_workflow_id=None
    for obj in comp.componants:

        if "oWorkflow" in obj.content :#and ( obj.action_required == 'U' or obj.action_required == 'C') :
            new_workflow_id = obj.content["trg_id"]
            old_workflow_id=obj.content["oWorkflow"]["id"]

            new_workflow_id = create_oWorkFlows(obj, target_id, onyx_trg)
            if new_workflow_id is None:
                new_workflow_id = obj.content["trg_id"]
            for obj in comp.componants:
                if "oWorkflowStep" in obj.content :
                    if obj.content["oWorkflowStep"]["oWorkflowId"] == old_workflow_id:

                        if obj.content["oWorkflowStep"]["jobType"] == 1:
                            pass

                        if obj.content["oWorkflowStep"]["jobType"] == 2:
                            pass

                        if obj.content["oWorkflowStep"]["jobType"] == 3: #SQL SCRIPT
                            add_to_workflow_sqlscript(obj,onyx_src,onyx_trg,tenant_src,tenant_trg,source_id,target_id,new_workflow_id)

                        if obj.content["oWorkflowStep"]["jobType"] == 4: #SCRIPT_SHELL
                            add_to_workflow_shellscript(obj,onyx_src,onyx_trg,tenant_src,tenant_trg,source_id,target_id,new_workflow_id)

                        if obj.content["oWorkflowStep"]["jobType"] == 5:
                            pass

                        if obj.content["oWorkflowStep"]["jobType"] == 6:
                            add_to_workflow_notification(obj,onyx_src,onyx_trg,tenant_src,tenant_trg,source_id,target_id,new_workflow_id)
    return new_workflow_id
def create_oWorkFlows(obj, id, onyx_trg: NxOnyxApi):
    id = str(id)

    if "oWorkflow" in obj.content and obj.action_required == 'U' or obj.action_required == 'C':
        data = obj.content["oWorkflow"]
        response= onyx_trg.createWorkflow(id = obj.content["trg_id"],name=data["name"], oProjectId=id,
                                enableCrossProject=data["enableCrossProject"],
                                emailSentOnError=data["emailSentOnError"], emailRecipients=data["emailRecipients"])
        return response


from rich.console import Console
import os

def get_tenantname():
    console = Console()
    env_conf_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../env.conf"))


    # Check if the env.conf file is empty
    if os.path.getsize(env_conf_path) == 0:
        console.print("No tenants are set yet. It seems your env.conf is empty.", style="yellow")
        console.print("Please configure your tenant parameters using the tenant set command.", style="yellow")
        return

    with open(env_conf_path, 'r') as f:
        line_number = 1
        for line in f:
            line = line.strip()
            if line.startswith('#'):
                console.print(f"{line_number}. {line[1:].upper()}", style="yellow")
                line_number += 1


import os
from rich.console import Console
from LIBS_CLI.init_envconf import init_envconf
def exist_envconf():
    filepath=os.path.abspath(os.path.join(os.path.dirname(__file__), "../env.conf"))
    if os.path.isfile(filepath) == False:
        console = Console()
        console.print("Please configure the tenant parameters before executing this command.", style="red")
        return False


import os

def ask_package(folder):
    print("List of available packages : ")
    packages = os.listdir(folder)
    choice_num = 0
    name = ""
    for package in packages:
        if ".zip" in package:
            choice_num += 1
            print("Choix n°{} : {}".format(str(choice_num).ljust(2), package).ljust(60))
    selected_num = input("\nSaisissez le numéro de package à importer : ")
    choice_num = 0
    for package in packages:
        if ".zip" in package:
            choice_num += 1
            if str(choice_num) == str(selected_num):
                name = package
    print("\nLe package \"{}\" a été sélectionné.\n".format(name))
    return name


import os
from rich.console import Console
from CLASSES.NxTenant import NxTenant
from LIBS_CLI.exist_tenantName import exist_tenantname
from LIBS_CLI.refactor_dict import refactor_dict
from LIBS_CLI.creer_arborescence import creer_arborescence
from CLASSES.NxOnyxApi import NxOnyxApi

def action_4(tenant,folder):
    console = Console()
    #DOWNLOAD
    new_folder=folder+'/'+tenant
    if os.path.exists(new_folder):
        #shutil.rmtree("TENANTS/SRC", onerror=on_rm_error)
        console.print("File named " + tenant + " already exist in " + folder, style="red")
    elif exist_tenantname(tenant)!=True:
        console.print("Tenant name "+tenant+" does not exist !", style="red")
    else:
        localRepositoryFolder = new_folder
        tenant_src = NxTenant(domain=os.environ[tenant + '_TENANTDOMAIN'], tenant_id=os.environ[tenant + '_TENANTID'],
                              tenant_name=os.environ[tenant + '_TENANTNAME'],
                              username=os.environ[tenant + '_TENANTUSERNAME'], password=os.environ[tenant + '_TENANTPASSWORD'])
        onyx_src = NxOnyxApi(tenant_src.domain, tenant_src.username, tenant_src.password, tenant_src.tenant_id)
        tree = onyx_src.getTree()

        dict=refactor_dict(tree)
        creer_arborescence(dict,onyx_src,localRepositoryFolder)





from .EnvConfElement import EnvConfElement
from .NxComponant import NxComponant
from .NxComponantAttributes import NxComponantAttributes
from .NxOnyxApi import *
from .NxProject import *
from .NxTenant import *





from CLASSES.NxProject import NxProject
from CLASSES.NxComponantAttributes import NxComponantAttributes
from LIBS_BUS.apply_transco import apply_transco

def compare_onyx_projects(project_pkg: NxProject, project_tnt: NxProject):
    project_result = NxProject(project_tnt.id, project_tnt.content, project_tnt.name, [])
    transco_matrix = []
    comparison_matrix = {
        "connection": NxComponantAttributes("oConnection", ["connectionType", "isWritable", "name", "documentation"]),
        "fileprovider": NxComponantAttributes("oFileProvider", ["connectionString", "containerName", "name", "organizationUnitId","type"]),
        "sqlscript": NxComponantAttributes("osqlScript", ["query", "oConnectionName","oConnectionId"]),
        "shellscript": NxComponantAttributes("oShellScript", ["packages", "script", "type"]),
        "notification": NxComponantAttributes("oNotification", ["type", "subject", "body", "recipients"]),
        "variable": NxComponantAttributes("oVariable", ["name", "description", "value", "query", "oVariableType", "oConnectionId"]),
        "pipeline": NxComponantAttributes("oDatapipeline", ["sourceConnectionId", "sourceConnectionType", "destinationConnectionId", "destinationConnectionType", "sourceTable", "destinationTable", "queryFilter"]),
        "pipeline_column": NxComponantAttributes("oColumnDatapipeline", ["oFlowId", "sourceColumnName", "destinationColumnDataType", "sourceCharacterMaximumLength", "sourceColumnNumericPrecision", "sourceColumnNumericScale", "sourceColumnDatatype", "ordinal"]),
        "form": NxComponantAttributes("oForm", ["displayedName", "documentation", "isActive","oConnectionId"]),
        "form_column": NxComponantAttributes("oFormColumn", ["displayedName", "type", "isNullable","displayOrder", "isDisplayed", "isDropDown", "dropDownQuery", "characterMaximumLength", "numericPrecision", "numericScale", "editorType", "editorConfiguration"]),
        "workflow": NxComponantAttributes("oWorkflow", ["oProjectId", "name","enableCrossProject", "emailSentOnError", "emailRecipients"]),
        "workflow_step": NxComponantAttributes("oWorkflowStep", ["name","jobType", "stepOrder", "isActive", "stopWorkflowOnError"]),
        "report": NxComponantAttributes("oReport", ["configuration", "name", "oProjectId", "documentation", "configuration"]),
        "widget": NxComponantAttributes("oWidget", ["query", "configuration", "oConnectionId", "oWorkflowId", "oFormId", "oFileProviderId", "type"]),
        "widget_filter": NxComponantAttributes("oWFilter", ["description", "query", "oConnectionId", "type", "value"]),
        "widget_button": NxComponantAttributes("oWButton", ["configuration", "actionType", "osqlScriptId"]),
        "schedules": NxComponantAttributes("oSchedule", ["name", "cronFormula", "isActive"])
    }

    for key, comp_attributes in comparison_matrix.items():
        for pkg_componant in project_pkg.componants:
            if pkg_componant.object_type == key:
                pkg_componant.content["trg_id"]=None
                object_exist = False
                for tnt_componant in project_tnt.componants:
                    if tnt_componant.object_type == key:
                        if pkg_componant.primary_key == tnt_componant.primary_key:
                            object_exist = True
                            matching_component = tnt_componant

                            if comp_attributes.json_object == None:
                                transco_matrix.append({"package_object_id": pkg_componant.content["id"], "tenant_object_id": tnt_componant.content["id"]})
                                pkg_componant.content["trg_id"]=tnt_componant.content["id"]
                            else:
                                transco_matrix.append({"package_object_id": pkg_componant.content[comp_attributes.json_object]["id"], "tenant_object_id": tnt_componant.content[comp_attributes.json_object]["id"]})
                                pkg_componant.content["trg_id"]=tnt_componant.content[comp_attributes.json_object]["id"]
                if object_exist:
                    diff = obj_diff(pkg_componant.content, matching_component.content, comp_attributes.json_object, comp_attributes.comparable_attributes, transco_matrix)
                    diff[:] = [objet for objet in diff if not objet.endswith("Id")]
                    if len(diff) > 0:
                        pkg_componant.action_required = "U"
                        pkg_componant.list_of_diff = ','.join(diff)
                    else:
                        pkg_componant.action_required = "N"
                else:
                    pkg_componant.action_required = "C"
                project_result.componants.append(pkg_componant)

    return project_result



def obj_diff(pkg, tnt, obj, tag_list, transco_matrix):
    res = []
    for tag in tag_list:
        if obj is None:
            val_pkg = apply_transco(transco_matrix, str(pkg[tag]))
            val_tnt = str(tnt[tag])
        else:
            val_pkg = apply_transco(transco_matrix, str(pkg[obj][tag]))
            val_tnt = str(tnt[obj][tag])
        if not val_pkg == val_tnt:

            res.append(tag)
    return res



from CLASSES import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS.read_onyx_project_api import read_onyx_project_api
from LIBS_BUS.compare_onyx_projects import compare_onyx_projects
from LIBS_CLI.cli_get_organisation_unit import cli_get_organisation_unit_name, cli_get_organisation_unit_id


def deploy_oFileProviders(onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi, tenant_src: NxTenant,
                      tenant_trg: NxTenant, source_id, target_id, script_type,name):

    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type,name=name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type,name=name)

    comp = compare_onyx_projects(project_src, project_tnt)
    response = None
    for obj in comp.componants:


        if "oFileProvider" in obj.content and obj.action_required == 'U' or obj.action_required == 'C':
            data = obj.content["oFileProvider"]
            ou_name = cli_get_organisation_unit_name(onyx_src, data["organizationUnitId"])
            ou_id = cli_get_organisation_unit_id(onyx_trg, ou_name)
            if ou_id is not None:
                response=onyx_trg.createFileProvider(id=obj.content["trg_id"],connectionString=data["connectionString"], containerName=data["containerName"],name=data["name"], organizationUnitId=ou_id, type=data["type"])

    return response



def apply_transco(transco_matrix, txt: str):
    res: str = txt
    if res is not None:
        for transco in transco_matrix:
            res = res.replace(transco["package_object_id"], transco["tenant_object_id"])
    return res


from LIBS_BUS import find_onyx_project
from LIBS_CLI import cli_create_tenant, cli_connect_api


def check_project_existence(source_tenant, target_tenant, project):
    status=True

    tenant_src = cli_create_tenant(source_tenant)
    onyx_src = cli_connect_api(tenant_src)
    # Check if the project exists in the source tenant
    project_id_source = find_onyx_project(onyx_src, project)


    tenant_trg = cli_create_tenant(target_tenant)
    onyx_trg = cli_connect_api(tenant_trg)
    # Check if the project exists in the source tenant
    project_id_target = find_onyx_project(onyx_trg, project)


    if project_id_source is None:
        print(f"Project '{project}' does not exist in the source tenant '{source_tenant}'.")
        status= False

    if project_id_target is None:
        print(f"Project '{project}' does not exist in the target tenant '{target_tenant}'.")
        status= False

    return status


from CLASSES import NxOnyxApi


def cli_get_id_sqlScript(script_name,id,onyx_trg: NxOnyxApi):

    get_sqlScripts = onyx_trg.getSqlScriptsByProject(id)

    for element in get_sqlScripts["items"]:
        if element["osqlScript"]["name"] == script_name:
            sqlScript_id = element["osqlScript"]["id"]
            return sqlScript_id

def cli_get_id_shellScript(script_name,id,onyx_trg: NxOnyxApi):

    get_shellScripts = onyx_trg.getShellScriptsByProject(id)

    for element in get_shellScripts["items"]:
        if element["oShellScript"]["name"] == script_name:
            shellScript_id = element["oShellScript"]["id"]
            return shellScript_id

def cli_get_id_notification(script_name,id,onyx_trg: NxOnyxApi):

    get_notifications = onyx_trg.getNotificationsByProject(id)

    for element in get_notifications["items"]:
        if element["oNotification"]["name"] == script_name:
            notification_id = element["oNotification"]["id"]
            return notification_id

def cli_get_id_form(script_name,id,onyx_trg: NxOnyxApi):
    get_form = onyx_trg.getFormsByProject(id)

    for element in get_form["items"]:
        if element["oForm"]["technicalName"] == script_name:
            form_id = element["oForm"]["id"]
            return form_id


import os
import json
import shutil
from CLASSES.NxProject import NxProject

def write_to_disk_onyx_project(project: NxProject, folder, filename):
    print("Ecriture du package du projet : {}".format(project.name))

    # Création du dossier racine du projet
    temp_folder = folder + "write_to_disk_tempfolder"
    if os.path.exists(temp_folder):
        print("Deleting folder: "+temp_folder)
        shutil.rmtree(temp_folder)
    os.makedirs(temp_folder)

    # Ecriture du fichier descriptif du projet
    file = open(temp_folder + "\\project.json", 'w', encoding="utf-8")
    file.write(json.dumps(project.json, indent=4, ensure_ascii=False))
    file.close()

    # Création de l'aborescence de dossiers du projet et écriture des composants
    for componant in project.componants:
        component_folder = temp_folder + "\\" + componant.object_type
        if not os.path.exists(component_folder):
            os.makedirs(component_folder)
        file = open(component_folder + "\\" + componant.primary_key + ".json", 'w', encoding="utf-8")
        file.write(json.dumps(componant.content, indent=4, ensure_ascii=False))
        file.close()

    # Création du package zip
    shutil.make_archive(folder + "\\" + filename, 'zip', temp_folder)
    shutil.rmtree(temp_folder)


from CLASSES.NxOnyxApi import NxOnyxApi
def cli_get_oWorkflowId(data,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):


    workflow_name=onyx_src.getWorkflow(data["oWorkflowId"])
    get_workflows = onyx_trg.getWorkflowsByProject(id)["items"]

    for element in get_workflows:

        if element["oWorkflow"]["name"] == workflow_name["oWorkflow"]["name"]:
            workflow_id = element["oWorkflow"]["id"]
            return workflow_id

def cli_get_oFormId(obj,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):
    data = onyx_src.getWidget(obj.content["oWidget"]["id"])
    data = data["oWidget"]
    form_name = onyx_src.getForm(data["oFormId"])
    get_forms = onyx_trg.getFormsByProject(id)

    for element in get_forms:
        if element["oForm"]["name"] == form_name["oForm"]["name"]:
            form_id = element["oForm"]["id"]
            return form_id

def cli_get_oFileProviderId(obj,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):
    data = onyx_src.getWidget(obj.content["oWidget"]["id"])
    data = data["oWidget"]
    FileProvider_name = onyx_src.getFileProvider(data["oFileProviderId"])
    get_FileProviders = onyx_trg.getFileProvidersByProject(id)

    for element in get_FileProviders:
        if element["oFileProvider"]["name"] == FileProvider_name["oFileProvider"]["name"]:
            FileProvider_id = element["oFileProvider"]["id"]
            return FileProvider_id

def cli_get_oSqlScriptId(data,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):

    SqlScript_name = onyx_src.getSqlScript(data["osqlScriptId"])
    get_SqlScripts = onyx_trg.getSqlScriptsByProject(id)["items"]

    for element in get_SqlScripts:

        if element["osqlScript"]["name"] == SqlScript_name["osqlScript"]["name"]:
            SqlScript_id = element["osqlScript"]["id"]
            return SqlScript_id

def cli_get_workflow_name(onyx :NxOnyxApi,id):
    data=onyx.getWorkflow(id)["oWorkflow"]["name"]
    return data


def cli_get_fileprovider_name(onyx :NxOnyxApi,id):
    data=onyx.getFileProvider(id)["oFileProvider"]["name"]
    return data

def cli_get_form_name(onyx :NxOnyxApi,id):
    data=onyx.getForm(id)["oForm"]["name"]
    return data

def cli_get_sql_name(onyx :NxOnyxApi,id):
    data=onyx.getForm(id)["osqlScript"]["name"]
    return data


import json

class NxComponant:
    def __init__(self, object_type: str, primary_key: str, content:json, action_required: str, list_of_diff: str):
        self.object_type = object_type
        self.primary_key = primary_key
        self.content = content
        self.action_required = action_required
        self.list_of_diff = list_of_diff
        self.trg_id= None



def refactor_dict(donnees, parent_id=None):
    sous_projets = []
    for element in donnees:
        if element['categoryId'] == parent_id:
            sous_projet = element
            sous_projet['subproject'] = refactor_dict(donnees, element['id'])
            sous_projets.append(sous_projet)
    return sous_projets




from alive_progress import alive_bar
from CLASSES.NxOnyxApi import NxOnyxApi
from CLASSES.NxProject import NxProject
from CLASSES.NxComponant import NxComponant

def read_onyx_project_api(onyx: NxOnyxApi, id, tenant_name,object,name):
    #illegal_characters = ["#","%","&","{","}","\\","<",">","*","?","/","$"," ","!","'","""""",":","@","+","`", "|", "="]
    #illegal_characters = """#%&{}\\<>*?/$ !'":@+,`|="""
    illegal_characters = ""
    exported_project = NxProject("", "", "", [])
    exported_project.id = id
    exported_project.json = onyx.getProject(id)
    exported_project.name = exported_project.json["oProject"]["name"]
    print("Downloading project \"{}\" from tenant \"{}\"".format(exported_project.name, tenant_name))

    if object=="ALL":
        i=12
    else:
        i=1

    with alive_bar(i) as bar:
        if object.upper()=="CONNECTION" or object=="ALL":
            for obj in onyx.getConnectionsByProject(id):
                if obj["oConnection"]["shortName"]==name or name == "ALL":
                    component = NxComponant("connection", obj["oConnection"]["shortName"].translate(str.maketrans('', '', illegal_characters)), onyx.getConnection(obj["oConnection"]["id"]), "", "")
                    exported_project.componants.append(component)
            bar()

        if object.upper() == "FILE_PROVIDER" or object=="ALL":
            for obj in onyx.getFileProvidersByProject(id):
                if obj["oFileProvider"]["name"] == name or name == "ALL":
                    component = NxComponant("fileprovider", obj["oFileProvider"]["name"].translate(str.maketrans('', '', illegal_characters)), onyx.getFileProvider(obj["oFileProvider"]["id"]), "", "")
                    exported_project.componants.append(component)
            bar()

        if object.upper() == "NOTIFICATION" or object=="ALL":
            for obj in onyx.getNotificationsByProject(id)["items"]:
                if obj["oNotification"]["name"] == name or name == "ALL":
                    component = NxComponant("notification", obj["oNotification"]["name"].translate(str.maketrans('', '', illegal_characters)), onyx.getNotification(obj["oNotification"]["id"]), "", "")
                    exported_project.componants.append(component)
            bar()

        if object.upper() == "REPORT" or object=="ALL":
            for obj in onyx.getReportsByProject(id)["items"]:
                if obj["oReport"]["name"] == name or name == "ALL":
                    component = NxComponant("report", obj["oReport"]["name"].translate(str.maketrans('', '', illegal_characters)), onyx.getReport(obj["oReport"]["id"]), "", "")
                    exported_project.componants.append(component)
            bar()

        if object.upper() == "SHELL_SCRIPT" or object=="ALL":
            for obj in onyx.getShellScriptsByProject(id)["items"]:
                if obj["oShellScript"]["name"] == name or name == "ALL":
                    component = NxComponant("shellscript", obj["oShellScript"]["name"].translate(str.maketrans('', '', illegal_characters)), onyx.getShellScript(obj["oShellScript"]["id"]), "", "")
                    exported_project.componants.append(component)

            bar()

        if object.upper() == "SQL_SCRIPT" or object=="ALL":

            for obj in onyx.getSqlScriptsByProject(id)["items"]:
                if obj["osqlScript"]["name"] == name or name == "ALL":
                    oConnectionName=onyx.getConnection(obj["osqlScript"]["oConnectionId"])["oConnection"]["shortName"]
                    data= onyx.getSqlScript(obj["osqlScript"]["id"])
                    data["osqlScript"]["oConnectionName"]=oConnectionName
                    component = NxComponant("sqlscript", obj["osqlScript"]["name"].translate(str.maketrans('', '', illegal_characters)), data, "", "")
                    exported_project.componants.append(component)
            bar()

        if object.upper() == "VARIABLE" or object=="ALL":
            for obj in onyx.getVariablesByProject(id)["items"]:
                if obj["oVariable"]["code"] == name or name == "ALL":
                    onyx.getVariable(obj["oVariable"]["id"])
                    component = NxComponant("variable", obj["oVariable"]["code"].translate(str.maketrans('', '', illegal_characters)), onyx.getVariable(obj["oVariable"]["id"]), "", "")
                    exported_project.componants.append(component)
            bar()

        if object.upper() == "SCHEDULES" or object=="ALL":
            for obj in onyx.getSchedulesByProject(id)["items"]:
                if obj["oSchedule"]["name"] == name or name == "ALL":
                    component = NxComponant("schedules", obj["oSchedule"]["name"].translate(str.maketrans('', '', illegal_characters)), onyx.getSchedule(obj["oSchedule"]["id"]), "", "")
                    exported_project.componants.append(component)
            bar()

        if object.upper() == "FORM" or object=="ALL":
            for obj in onyx.getFormsByProject(id)["items"]:
                if obj["oForm"]["technicalName"] == name or name == "ALL":

                    component = NxComponant("form", obj["oForm"]["technicalName"].translate(str.maketrans('', '', illegal_characters)), onyx.getForm(obj["oForm"]["id"]), "", "")
                    exported_project.componants.append(component)
                    for col in onyx.getFormColumns(obj["oForm"]["id"])["items"]:
                        col['oFormColumn']['oFormId']=obj["oForm"]["id"]
                        component = NxComponant("form_column", (obj["oForm"]["technicalName"] + "$$" + col["oFormColumn"]["technicalName"]).translate(str.maketrans('', '', illegal_characters)), col, "", "")
                        exported_project.componants.append(component)
            bar()

        if object.upper() == "WORKFLOW" or object=="ALL":
            for obj in onyx.getWorkflowsByProject(id)["items"]:
                if obj["oWorkflow"]["name"] == name or name == "ALL":
                    component = NxComponant("workflow", obj["oWorkflow"]["name"].translate(str.maketrans('', '', illegal_characters)), onyx.getWorkflow(obj["oWorkflow"]["id"]), "", "")
                    exported_project.componants.append(component)
                    for step in onyx.getWorkflowSteps(obj["oWorkflow"]["id"])["items"]:
                        step["oWorkflowStep"]['oWorkflowId']=obj['oWorkflow']['id']
                        component = NxComponant(object_type="workflow_step", primary_key=(obj["oWorkflow"]["name"] + "$$" + str(step["oWorkflowStep"]["stepOrder"])).translate(str.maketrans('', '', illegal_characters)),  content=step ,action_required="",list_of_diff="")
                        exported_project.componants.append(component)
            bar()

        if object.upper() == "DATAPIPELINE" or object=="ALL":
            for obj in onyx.getPipelinesByProject(id)["items"]:
                if obj["name"] == name or name == "ALL":
                    dataPipeline={"oDatapipeline":onyx.getPipeline(obj["id"])}
                    component = NxComponant("pipeline", obj["name"].translate(str.maketrans('', '', illegal_characters)), dataPipeline, "", "")
                    exported_project.componants.append(component)
                    for col in onyx.getPipelineColumnsIncluded(obj["id"])["items"]:
                        dataColumnPipeline={"oColumnDatapipeline":col}
                        component = NxComponant("pipeline_column", (obj["name"] + "$$" + col["destinationColumnName"]).translate(str.maketrans('', '', illegal_characters)), dataColumnPipeline, "", "")
                        exported_project.componants.append(component)
            bar()

        if object.upper() == "WIDGET" or object=="ALL":
            for obj in onyx.getWidgetsByProject(id)["items"]:
                if obj["oWidget"]["name"] == name or name == "ALL":

                    component = NxComponant("widget", obj["oWidget"]["name"].translate(str.maketrans('', '', illegal_characters)), onyx.getWidget(obj["oWidget"]["id"]), "", "")
                    exported_project.componants.append(component)
                    for filter in onyx.getWidgetFilters(obj["oWidget"]["id"])["items"]:

                        datafilter={"oWFilter":filter}
                        component = NxComponant("widget_filter", (obj["oWidget"]["name"] + "$$" + filter["name"]).translate(str.maketrans('', '', illegal_characters)), datafilter, "", "")
                        exported_project.componants.append(component)

                    for button in onyx.getWidgetRowButtons(obj["oWidget"]["id"])["items"]:

                        databutton={"oWButton":button}
                        component = NxComponant("widget_button", (obj["oWidget"]["name"] + "$$" + button["name"]).translate(str.maketrans('', '', illegal_characters)), databutton, "", "")
                        exported_project.componants.append(component)

            bar()

    return exported_project


from CLASSES import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects


def deploy_oNotifications(onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi, tenant_src: NxTenant,
                      tenant_trg: NxTenant, source_id, target_id, script_type,name):
    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type, name=name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type, name=name)
    comp = compare_onyx_projects(project_src, project_tnt)

    target_id = str(target_id)
    response=None

    for obj in comp.componants:
        response=obj.content["trg_id"]
        if "oNotification" in obj.content and obj.action_required == 'U' or obj.action_required == 'C':

            data = obj.content["oNotification"]
            response= onyx_trg.createNotification(id=obj.content["trg_id"],name=data["name"], oProjectId=target_id, recipients=data["recipients"],
                                        subject=data["subject"], body=data["body"], type=data["type"], tenant=tenant_trg.tenant_id)
    return response

    # /!\ DESTINATAIRES NON MISE A JOUR




import os
from CLASSES.NxTenant import NxTenant

def cli_create_tenant(tenant):
    tenant = NxTenant(domain=os.environ[tenant + '_TENANTDOMAIN'],
                      tenant_id=os.environ[tenant + '_TENANTID'],
                      tenant_name=os.environ[tenant + '_TENANTNAME'],
                      username=os.environ[tenant + '_TENANTUSERNAME'],
                      password=os.environ[tenant + '_TENANTPASSWORD'])
    return tenant


import click
from rich.table import Table
from rich.console import Console
from CLASSES.NxProject import NxProject
from LIBS_BUS.compare_onyx_projects import compare_onyx_projects
from LIBS_BUS.find_onyx_project import find_onyx_project
from LIBS_BUS.read_onyx_project_api import read_onyx_project_api
from LIBS_CLI.cli_connect_api import cli_connect_api
from LIBS_CLI.cli_create_tenant import cli_create_tenant

def action_2(source_tenant, target_tenant, project):
    console = Console()

    tenant_src = cli_create_tenant(source_tenant)
    onyx_src = cli_connect_api(tenant_src)
    tenant_trg = cli_create_tenant(target_tenant)
    onyx_trg = cli_connect_api(tenant_trg)


    id = find_onyx_project(onyx_src, project)
    project_src = read_onyx_project_api(onyx_src, id, tenant_name=(tenant_src.tenant_name), object="ALL",name="ALL")
    console.print("")
    id = find_onyx_project(onyx_trg, project)
    project_trg = read_onyx_project_api(onyx_trg, id, tenant_name=tenant_trg.tenant_name, object="ALL",name="ALL")
    #clear_screen()
    table = Table(title="Differences detected for the project: {}\n".format(project_trg.name))
    table.add_column("Type", style="cyan", justify="center")
    table.add_column("Name", style="magenta", justify="left")
    table.add_column("Action", style="magenta", justify="center")
    table.add_column("Details", style="magenta", justify="left")
    project_result: NxProject = compare_onyx_projects(project_src, project_trg)
    for componant in project_result.componants:

        if componant.action_required == "C":
            table.add_row(componant.object_type, componant.primary_key, "TO CREATE", "")
        if componant.action_required == "U":
            table.add_row(componant.object_type, componant.primary_key, "TO UPDATE",
                        "diverging field(s): {}".format(componant.list_of_diff))
    console.print(table)




from CLASSES import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects
from LIBS_CLI.deploy_oConnections import deploy_oConnections
from LIBS_CLI.deploy_oForms import deploy_oForms
from LIBS_CLI.deploy_oWidgetButton import deploy_oWidgetButton
from LIBS_CLI.deploy_oWidgetFilter import deploy_oWidgetFilter
from LIBS_CLI.deploy_oWorkFlows import deploy_oWorkFlows
from LIBS_CLI.deploy_oFileProviders import deploy_oFileProviders
from LIBS_CLI.cli_get_connection import cli_get_connection_widget, cli_get_connection_name
from LIBS_CLI.cli_get_info import cli_get_oWorkflowId, cli_get_oFormId, cli_get_oFileProviderId, \
    cli_get_workflow_name, cli_get_fileprovider_name, cli_get_form_name


def deploy_oWidgets(onyx_src : NxOnyxApi, onyx_trg : NxOnyxApi, tenant_src : NxTenant, tenant_trg : NxTenant, source_id, target_id, script_type, object_name):

    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type, name=object_name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type, name=object_name)
    comp = compare_onyx_projects(project_src, project_tnt)


    id = str(target_id)
    response = None
    list_of_filter = None
    list_of_button = None

    for obj in comp.componants:
        if obj.content.get('oWidget') is not None :

            data = obj.content["oWidget"]
            if data["oConnectionId"] is not None:
                connection_id=cli_get_connection_widget(data["oConnectionId"], id, onyx_src, onyx_trg)
                if connection_id is None:
                    object_name = cli_get_connection_name(obj.content["oWidget"]["oConnectionId"], onyx_src)
                    connection_id=deploy_oConnections(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, "CONNECTION",object_name)
            else:
                connection_id=data["oConnectionId"]


            #GET WORKFLOW ID
            if data["oWorkflowId"] is not None:

                object_name = cli_get_workflow_name(onyx_src, data["oWorkflowId"])

                workflow_id=deploy_oWorkFlows(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, "WORKFLOW", object_name)
            else:
                workflow_id = data["oWorkflowId"]


            #GET FILEPROVIDER ID
            if data["oFileProviderId"] is not None:
                object_name = cli_get_fileprovider_name(onyx_src, obj.content["oWidget"]["oFileProviderId"])
                fileprovider_id = deploy_oFileProviders(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, "FILE_PROVIDER", object_name)
            else:
                fileprovider_id = data["oFileProviderId"]


            #GET FORM ID
            if data["oFormId"] is not None:
                object_name = cli_get_form_name(onyx_src, obj.content["oWidget"]["oFormId"])
                form_id = cli_get_oFormId(obj, id, onyx_src, onyx_trg)
                if form_id is None:
                    form_id=deploy_oForms(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type,object_name)
            else:
                form_id = data["oFormId"]

            response=create_oWidget(obj,id,onyx_trg, connection_id, workflow_id, form_id, fileprovider_id)
            list_of_filter=deploy_oWidgetFilter(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type,"ALL", response, data["id"])
            list_of_button=deploy_oWidgetButton(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type,"ALL", response, data["id"])

    return {"widget" : response, "filters" : list_of_filter, "buttons" : list_of_button}


def create_oWidget(obj, id, onyx_trg: NxOnyxApi, connection_id, workflow_id, form_id, fileprovider_id):
    id = str(id)
    response=obj.content["trg_id"]
    if "oWidget" in obj.content and obj.action_required == 'U' or obj.action_required == 'C':
        data = obj.content["oWidget"]

        response = onyx_trg.createWidget(id=obj.content["trg_id"], oProjectId=id, name=data["name"],
                                         description=data["description"], query=data["query"],
                                         configuration=data["configuration"], oConnectionId=connection_id,
                                         oWorkflowId=workflow_id, oFormId=form_id,
                                         oFileProviderId=fileprovider_id, type=data["type"])
    return response



from CLASSES.NxTenant import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS.compare_onyx_projects import compare_onyx_projects
from LIBS_BUS.read_onyx_project_api import read_onyx_project_api
from LIBS_CLI.cli_get_connection import cli_get_connection_widget
from LIBS_CLI.deploy_oConnections import deploy_oConnections
from LIBS_CLI.cli_get_connection import cli_get_connection_name


def deploy_oWidgetFilter(onyx_src : NxOnyxApi, onyx_trg : NxOnyxApi, tenant_src : NxTenant, tenant_trg : NxTenant, source_id, target_id, script_type,object_name,newWidgetId,oldWidgetId):
    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type, name=object_name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type, name=object_name)
    comp = compare_onyx_projects(project_src, project_tnt)
    response=None
    lst=[]

    for obj in comp.componants:
        if "oWFilter" in obj.content:
            response = obj.content["trg_id"]
            if obj.action_required == 'U' or obj.action_required == 'C':

                data = obj.content["oWFilter"]

                conn_id=data["oConnectionId"]
                if data["oConnectionId"] is not None:
                    conn_id = cli_get_connection_widget(data["oConnectionId"], target_id, onyx_src, onyx_trg)
                    if conn_id is None:
                        object_name = cli_get_connection_name(data["oConnectionId"], onyx_src)
                        conn_id = deploy_oConnections(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id,
                                                      "CONNECTION", object_name)

                if oldWidgetId==data["oWidgetId"]:
                    response=onyx_trg.createWidgetFilter(id=obj.content["trg_id"],name=data["name"], description=data["description"],
                                        oConnectionId=conn_id,
                                        oWidgetId=newWidgetId, query=data["query"], type=data["type"])

            lst.append(response)
    return lst


from CLASSES.NxOnyxApi import NxOnyxApi
from rich.console import Console


def ask_project(onyx: NxOnyxApi, message):
    console = Console()
    # Récupération de l'arbre des projet
    tree = onyx.getTree()
    project_list = []
    for object in tree:
        if object["detailType"] == 'project':
            prj = {"id": object["projectId"], "name": object["name"]}
            project_list.append(prj)
    # Listing des projets disponibles à l'export
    choice_num = 0
    console.print(f"[magenta]{message}[/magenta]")
    line = ""
    i = 0
    for prj in project_list:
        i += 1
        choice_num += 1
        line += (f"[yellow]{choice_num}[/yellow] . [cyan]{prj['name']}[/cyan]\n")
        if i == 4:
            console.print(line)
            line = ""
            i = 0
    if i > 0:
        console.print(line)
    # Choix du projet à exporter
    choice_num = 0
    id = ""
    selected_num = input("\nType the project's number you want to analyse: ")
    for prj in project_list:
        choice_num += 1
        if str(choice_num) == str(selected_num):
            id = prj["id"]
    return id


import click
from LIBS_CLI.exist_tenantName import exist_tenantname

def cli_tenant_validation(source_tenant, target_tenant):
    # Check if source tenant exists
    if not exist_tenantname(source_tenant):
        click.echo(f"The source tenant '{source_tenant}' does not exist or is invalid.")
        click.echo("Please configure your parameters using the tenantSet command.")
        return False

    # Check if target tenant exists
    if not exist_tenantname(target_tenant):
        click.echo(f"The target tenant '{target_tenant}' does not exist or is invalid.")
        click.echo("Please configure your parameters using the tenantSet command.")
        return False

    return True



from CLASSES.NxOnyxApi import NxOnyxApi
from CLASSES.NxTenant import NxTenant

def cli_connect_api(tenant : NxTenant):

    onyx = NxOnyxApi(tenant.domain, tenant.username, tenant.password, tenant.tenant_id)

    return onyx


from CLASSES import NxProject, NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import compare_onyx_projects, read_onyx_project_api
from LIBS_CLI.deploy_oConnections import deploy_oConnections
from LIBS_CLI.cli_get_connection import cli_get_connection_variable, cli_get_connection_name


def deploy_oVariables( onyx_src : NxOnyxApi, onyx_trg : NxOnyxApi, tenant_src : NxTenant, tenant_trg : NxTenant, source_id, target_id, script_type, object_name):

    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type,name=object_name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type,name=object_name)
    comp = compare_onyx_projects(project_src, project_tnt)

    for obj in comp.componants:
        if obj.content["oVariable"]["oConnectionId"] is not None:
            conn_id = cli_get_connection_variable(obj.content["oVariable"]["id"], target_id, onyx_src,onyx_trg)
            if conn_id is None:
                object_name = cli_get_connection_name(obj.content["oVariable"]["oConnectionId"], onyx_src)
                conn_id=deploy_oConnections(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, "CONNECTION",object_name)
                create_oVariables(obj, target_id, onyx_trg, conn_id,tenant_trg.tenant_id)
            else:

                create_oVariables(obj, target_id, onyx_trg, conn_id,tenant_trg.tenant_id)
        else:
            conn_id = None

            create_oVariables(obj, target_id, onyx_trg, conn_id,tenant_trg.tenant_id)


def create_oVariables(obj, id, onyx_trg: NxOnyxApi,conn_id,tenant_id):
    id = str(id)
    response=None
    if "oVariable" in obj.content and obj.action_required == 'C' or obj.action_required == 'U':

        data = obj.content["oVariable"]
        response=onyx_trg.createVariable(id = obj.content["trg_id"], code=data["code"], name=data["name"], description=data["description"],
                                oConnectionId=conn_id,
                                oProjectId=id, oVariableType=data["oVariableType"], tenantId=tenant_id,
                                query=data["query"], value=data["value"])
    return response



import os
import json
from alive_progress import alive_bar
import shutil
from CLASSES.NxProject import NxProject
from CLASSES.NxComponant import NxComponant

def read_onyx_project_package(package_path, destination_path):
    print("Lecture du package : \"{}\"".format(package_path))
    ropp_result = NxProject("", "", "", [])
    with alive_bar(12, force_tty = True) as bar:
        project_folder = destination_path + "read_package_tempfolder"
        bar()

        if os.path.exists(project_folder):
            shutil.rmtree(project_folder)
        shutil.unpack_archive(package_path, project_folder, 'zip')
        bar()

        for folder in os.listdir(project_folder):
            if not folder == "project.json":
                for file in os.listdir(project_folder + "\\" + folder):
                    componant = NxComponant(folder, file[:-5], json.loads(cust_read(project_folder + "\\" + folder, file)), "", "")
                    ropp_result.componants.append(componant)
                bar()

    return ropp_result

def cust_read(folder_name, file_name):
    try:
        file = open(folder_name + "\\" + file_name, 'r', encoding = "utf-8")
        content = file.read()
        file.close()
        return content
    except Exception as e:
        message = "Echec de lecture du fichier {} | {}".format(folder_name + "\\" + file_name, str(e))
        print(message)




from CLASSES.NxTenant import NxTenant

def ask_tenant(message):
    tenants = [
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="5", tenant_name ='ADS', username ="api@alchimiedatasolutions.com", password ="Dense?Zookeeper?Twenty4"),
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="12", tenant_name ='BLUETEK', username ="api@alchimiedatasolutions.com", password ="Crown4?Armless?Chivalry"),
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="6", tenant_name ='HBE', username ="api@alchimiedatasolutions.com", password ="Scrabble?Pulsate?Showcase6"),
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="10", tenant_name ='HBE_PREPROD', username ="api@alchimiedatasolutions.com", password ="Livestock-Spoken6-Harmful"),
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="7", tenant_name ='HBE_PROD', username ="api@alchimiedatasolutions.com", password ="Overnight2-Fantasy-Recopy"),
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="11", tenant_name ='MET_UAT', username ="api@alchimiedatasolutions.com", password ="Unwatched9?Mangle?Veto"),
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="16", tenant_name ='MET_PROD', username ="api@alchimiedatasolutions.com", password ="Untitled-Upturned4-Peculiar"),
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="4", tenant_name ='SLOGIA', username ="api@alchimiedatasolutions.com", password ="Stuffing-Tiptop5-Chaplain"),
        NxTenant(domain ="https://onyx-back.azurewebsites.net", tenant_id ="14", tenant_name ='TERRA LACTA', username ="api@alchimiedatasolutions.com", password ="Quizzical-Propose-Dayroom1"),
    ]
    print(message)
    line = ""
    i = 0
    print("")
    for tnt in tenants:
        i += 1
        line += ("Id : {} Name : {}".format(tnt.tenant_id.ljust(5), tnt.tenant_name)).ljust(55)
        if i == 4:
            print(line)
            line = ""
            i = 0
    if i > 0:
        print(line)
    id = input("\nPlease, type the tenant id: ")

    for tnt in [x for x in tenants if x.tenant_id == str(id)]:
        tenant = tnt
    print("\nTenant \"{}\" has been selected.\n".format(tenant.tenant_name))
    return tenant


from CLASSES import NxOnyxApi, NxTenant
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects
from LIBS_CLI.deploy_oSqlScripts import deploy_oSqlScripts
from LIBS_CLI.deploy_oWorkFlows import deploy_oWorkFlows
from LIBS_CLI.cli_get_info import cli_get_oWorkflowId, cli_get_oSqlScriptId, cli_get_sql_name, \
    cli_get_workflow_name
from LIBS_CLI.deploy_oConnections import deploy_oConnections
from LIBS_CLI.cli_get_connection import cli_get_connection_widget, cli_get_connection_name


def deploy_oWidgetButton(onyx_src : NxOnyxApi, onyx_trg : NxOnyxApi, tenant_src : NxTenant, tenant_trg : NxTenant, source_id, target_id,  script_type,object_name, newWidgetId,oldWidgetId):

    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type, name=object_name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type, name=object_name)
    comp = compare_onyx_projects(project_src, project_tnt)
    response = None
    lst=[]
    for obj in comp.componants:
        response=obj.content["trg_id"]
        if "oWButton" in obj.content and (obj.action_required == 'U' or obj.action_required == 'C'):
            data = obj.content["oWButton"]

            # GET WORKFLOW ID
            if data["oWorkflowId"] is not None:
                object_name=cli_get_workflow_name(onyx_src, obj.content["oWButton"]["oWorkflowId"])
                workflow_id = cli_get_oWorkflowId(obj.content["oWButton"], target_id, onyx_src, onyx_trg)
                if workflow_id is None:
                    workflow_id = deploy_oWorkFlows(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id,
                                                    "WORKFLOW", object_name)
            else:
                workflow_id = data["oWorkflowId"]

            # GET SQL_SCRIPT ID
            if data["osqlScriptId"] is not None:
                object_name = cli_get_sql_name(onyx_src, obj.content["oWButton"]["osqlScriptId"])
                script_id = cli_get_oSqlScriptId(obj.content["oWButton"], target_id, onyx_src, onyx_trg)
                if script_id is None:
                    script_id = deploy_oSqlScripts(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id,
                                                    "SQL_SCRIPT", object_name)
            else:
                script_id = data["osqlScriptId"]

            if oldWidgetId==data["oWidgetId"]:
                response=onyx_trg.createWidgetRowButton(id=obj.content["trg_id"],actionType=data["actionType"], configuration=data["configuration"],
                                               description=data["description"],
                                               name=data["name"], oSqlScriptId=script_id, oWidgetId=newWidgetId,
                                               oWorkflowId=workflow_id)
                lst.append(response)
    return response


import shutil
import stat
import os


#GRANT PERMISSIONS TO DELETE FILES
def on_rm_error( func, path, exc_info):
    # path contains the path of the file that couldn't be removed
    # let's just assume that it's read-only and unlink it.
    os.chmod( path, stat.S_IWRITE )
    os.unlink( path )


import os
from datetime import datetime
from rich.table import Table
from rich.console import Console
from rich.panel import Panel

def clear_screen():
    os.system('cls')
    console = Console()
    grid = Table.grid(expand=True)
    grid.add_column(justify="left")
    grid.add_column(justify="center")
    grid.add_column(justify="right")
    grid.add_row("Product version : 0.4",
                 "[b]ONYX[/b] : Project Life Cycle Management",
                 datetime.now().ctime().replace(":", "[blink]:[/]"),
                 )
    console.print(Panel(grid))




from .action_1 import action_1
from .action_2 import action_2
from .action_3 import action_3
from .action_4 import action_4
from .action_5 import action_5
from .ask_package import ask_package
from .ask_project import ask_project
from .ask_tenant import ask_tenant
from .cli_add_to_workflow import add_to_workflow_sqlscript, add_to_workflow_notification, add_to_workflow_shellscript
from .cli_connect_api import cli_connect_api
from .cli_create_tenant import cli_create_tenant
from .cli_get_connection import cli_get_connection_sql, cli_get_connection_variable
from .cli_get_id import cli_get_id_form, cli_get_id_notification, cli_get_id_sqlScript, cli_get_id_shellScript
from .cli_get_info import  cli_get_oWorkflowId, cli_get_oFormId, cli_get_oSqlScriptId, cli_get_oFileProviderId
from .cli_get_organisation_unit import *
from .cli_tenant_validation import cli_tenant_validation
from .creer_arborescence import creer_arborescence
from .deploy_oConnections import deploy_oConnections
from .deploy_oFileProviders import deploy_oFileProviders
from .deploy_oForms import create_oForms, deploy_oForms, create_formColumn
from .deploy_oNotifications import deploy_oNotifications
from .deploy_oReports import deploy_oReports
from .deploy_oShellScripts import deploy_oShellScripts
from .deploy_oSqlScripts import create_oSqlScripts, deploy_oSqlScripts
from .deploy_oVariables import deploy_oVariables, create_oVariables
from .deploy_oWidgets import deploy_oWidgets
from .deploy_oWorkFlows import deploy_oWorkFlows, create_oWorkFlows
from .deploy_oWorkFlowStep import deploy_oWorkFlowStep
from .display_env import display_env
from .exist_envconf import exist_envconf
from .exist_tenantName import exist_tenantname
from .get_connection_name import get_connection_name
from .get_envconf import get_envconf
from .get_tenantname import get_tenantname
from .init_envconf import init_envconf
from .list_project import list_project
from .load_envconf import load_envconf
from .on_rm_error import on_rm_error
from .parameter_selector import paramter_selector
from .print_console import print_console
from .print_menu import print_menu
from .refactor_dict import refactor_dict
from .transco_id import transco_id
from .update_envconf import update_envconf
from .exist_tenantName import exist_tenantname




from CLASSES import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects
from LIBS_CLI.deploy_oConnections import deploy_oConnections
from LIBS_CLI.cli_get_connection import cli_get_connection_form, cli_get_connection_name
from LIBS_CLI.cli_get_id import cli_get_id_form


def deploy_oForms(onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi, tenant_src: NxTenant, tenant_trg: NxTenant, source_id, target_id, script_type,name):
    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type, name=name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type, name=name)
    comp = compare_onyx_projects(project_src, project_tnt)

    new_form_id=None
    for obj in comp.componants:
        new_form_id=obj.content["trg_id"]
        if "oForm" in obj.content:
            old_form_id = obj.content["oForm"]["id"]

            conn_id = cli_get_connection_form(obj.content["oForm"]["id"], target_id, onyx_src,onyx_trg)
            if conn_id is None:
                object_name=cli_get_connection_name(obj.content["oForm"]["oConnectionId"],onyx_src)
                conn_id=deploy_oConnections(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, "CONNECTION", object_name)

            new_form_id = create_oForms(obj, target_id, onyx_trg, conn_id)
            if new_form_id is None:
                new_form_id = cli_get_id_form(obj.content["oForm"]["displayedName"], target_id, onyx_trg)

            create_formColumn(onyx_trg, comp, old_form_id, new_form_id)

    return new_form_id
def create_oForms(obj, id, onyx_trg: NxOnyxApi, conn_id):
    id = str(id)
    response = None
    if "oForm" in obj.content and obj.action_required == 'U' or obj.action_required == 'C':

        data = obj.content["oForm"]
        response = onyx_trg.createForm(id=obj.content["trg_id"],displayedName= data["displayedName"], documentation=data["documentation"], oProjectId=id,
                                    oConnectionId=conn_id, technicalName= data["technicalName"], isActive=data["isActive"])
    return response


def create_formColumn(onyx_trg: NxOnyxApi, comp, old_form_id, new_form_id):

    for obj in comp.componants:
        if "oFormColumn" in obj.content and (obj.action_required == 'U'  or obj.action_required == 'C'):
            if obj.content["oFormColumn"]["oFormId"] == old_form_id:
                data = obj.content["oFormColumn"]
                onyx_trg.createFormColumn(id=obj.content["trg_id"],characterMaximumLength=data["characterMaximumLength"],
                                          displayOrder=data["displayOrder"],
                                          displayedName=data["displayedName"], dropDownQuery=data["dropDownQuery"],
                                          editorType=data["editorType"],
                                          isDisplayed=data["isDisplayed"], isDropDown=data["isDropDown"],
                                          isNullable=data["isNullable"],
                                          numericPrecision=data["numericPrecision"],
                                          technicalName=data["technicalName"], numericScale=data["numericScale"],
                                          oFormId=new_form_id, type=data["type"])



from CLASSES.NxOnyxApi import NxOnyxApi

"""def cli_get_connection_sql(obj,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):

    data = onyx_src.getSqlScript(obj.content["osqlScript"]["id"])
    data = data["osqlScript"]
    connection_name=onyx_src.getConnection(data["oConnectionId"])
    get_connections = onyx_trg.getConnectionsByProject(id)

    for element in get_connections:
        if element["oConnection"]["name"] == connection_name["oConnection"]["name"]:
            connection_id = element["oConnection"]["id"]
            return connection_id"""

"""def cli_get_connection_variable(obj,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):

    data = onyx_src.getVariable(obj.content["oVariable"]["id"])
    data = data["oVariable"]
    connection_name=onyx_src.getConnection(data["oConnectionId"])
    get_connections = onyx_trg.getConnectionsByProject(id)

    for element in get_connections:
        if element["oConnection"]["name"] == connection_name["oConnection"]["name"]:
            connection_id = element["oConnection"]["id"]
            return connection_id"""
def cli_get_connection_variable(obj,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):

    data = onyx_src.getVariable(obj)
    data = data["oVariable"]
    connection_name=onyx_src.getConnection(data["oConnectionId"])
    get_connections = onyx_trg.getConnectionsByProject(id)

    for element in get_connections:
        if element["oConnection"]["name"] == connection_name["oConnection"]["name"]:
            connection_id = element["oConnection"]["id"]
            return connection_id

def cli_get_connection_widget(data,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):
    connection_name=onyx_src.getConnection(data)
    get_connections = onyx_trg.getConnectionsByProject(id)

    for element in get_connections:
        if element["oConnection"]["name"] == connection_name["oConnection"]["name"]:
            connection_id = element["oConnection"]["id"]
            return connection_id

def cli_get_connection_sql(obj,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):

    data = onyx_src.getSqlScript(obj)
    data = data["osqlScript"]
    connection_name=onyx_src.getConnection(data["oConnectionId"])
    print("sql", connection_name)
    get_connections = onyx_trg.getConnectionsByProject(id)

    for element in get_connections:
        if element["oConnection"]["name"] == connection_name["oConnection"]["name"]:
            connection_id = element["oConnection"]["id"]
            return connection_id

def cli_get_connection_form(obj,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):

    data = onyx_src.getForm(obj)
    print("cli_get_connection_form",data)
    data = data["oForm"]
    connection_name=onyx_src.getConnection(data["oConnectionId"])
    print(connection_name)
    get_connections = onyx_trg.getConnectionsByProject(id)

    for element in get_connections:
        if element["oConnection"]["name"] == connection_name["oConnection"]["name"]:
            connection_id = element["oConnection"]["id"]
            return connection_id

def cli_get_connection_name(id,onyx_src : NxOnyxApi):
    data=onyx_src.getConnection(id)
    return data["oConnection"]["shortName"]

def cli_get_connection_datapipeline(data,id,onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):

    connection_name=onyx_src.getConnection(data)
    get_connections = onyx_trg.getConnectionsByProject(id)

    for element in get_connections:
        if element["oConnection"]["name"] == connection_name["oConnection"]["name"]:
            connection_id = element["oConnection"]["id"]
            return connection_id


def cli_get_connection_report(obj, report_data, project_id, onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi):
    # Get the report data from the source Onyx instance

    # Extract connection name from the report data
    connection_name = onyx_src.getConnection(obj)

    # Get connections associated with the project from the target Onyx instance
    target_connections = onyx_trg.getConnectionsByProject(project_id)

    # Iterate through connections to find a match based on connection name
    for connection in target_connections:
        if connection["oConnection"]["name"] == connection_name["oConnection"]["name"]:
            # Return the ID of the matching connection
            return connection["oConnection"]["id"]



import os
import dotenv
from CLASSES.EnvConfElement import EnvConfElement

def update_envconf(ele:EnvConfElement):
    os.environ[ele.key] = ele.value
    dotenv.set_key(ele.filename, ele.key, os.environ[ele.key])




from CLASSES import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects
import json

from LIBS_CLI.cli_get_connection import cli_get_connection_report
from LIBS_CLI.deploy_oWidgets import deploy_oWidgets


def deploy_oReports(onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi, tenant_src : NxTenant, tenant_trg : NxTenant, source_id, target_id, script_type,object_name):

    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type, name=object_name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type, name=object_name)
    comp = compare_onyx_projects(project_src, project_tnt)

    id = str(target_id)

    for obj in comp.componants:

        if "oReport" in obj.content and obj.action_required == 'U' or obj.action_required == 'C':
            data = obj.content["oReport"]
            if data["configuration"] is not None:
                configuration= json.loads(data["configuration"])
                for config in configuration:
                    # Call replace_widget() function to get the new values
                    new_widgetId = replace_widget(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id,
                                             source_widget_id=config["id"])

                    # Update the values in the config dictionary
                    config["id"] = new_widgetId["widget"]

                    config["inputs"]["widgetId"] = new_widgetId["widget"]
                    config["inputs"]["reportId"] = obj.content["trg_id"]

                    i=0
                    for filter in config["filters"]:
                        # Call cli_get_connection_report() to get the new connection ID
                        if filter["oConnectionId"] is not None:
                            new_connection_id = cli_get_connection_report(filter["oConnectionId"], data, target_id, onyx_src, onyx_trg)
                        else:
                            new_connection_id=None
                        # Update the oConnectionId in the filter
                        filter["oConnectionId"] = new_connection_id
                        filter["id"] = new_widgetId["filters"][i]
                        i+=1

            #4. Deploy_report
            new_configuration=json.dumps(configuration)
            onyx_trg.createReport(id=obj.content["trg_id"], name=data["name"], oProjectId=id,documentation=data["documentation"],configuration=new_configuration)
def get_widget_name_from_id(onyx_src: NxOnyxApi, source_widget_id):
    data = onyx_src.getWidget(source_widget_id)
    return data["oWidget"]["name"]


def replace_widget(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id,source_widget_id):
    # Call function to get widget name from source widget ID
    widget_name = get_widget_name_from_id(onyx_src, source_widget_id)
    response = deploy_oWidgets(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, "WIDGET", widget_name)
    return response





from rich.console import Console
from LIBS_BUS.find_onyx_project import find_onyx_project
from LIBS_CLI.deploy_oDataPipelines import deploy_oDataPipelines
from LIBS_CLI.deploy_oReports import deploy_oReports
from LIBS_CLI.cli_connect_api import cli_connect_api
from LIBS_CLI.cli_create_tenant import cli_create_tenant
from LIBS_CLI.deploy_oConnections import deploy_oConnections
from LIBS_CLI.deploy_oFileProviders import deploy_oFileProviders
from LIBS_CLI.deploy_oForms import deploy_oForms
from LIBS_CLI.deploy_oNotifications import deploy_oNotifications
from LIBS_CLI.deploy_oShellScripts import deploy_oShellScripts
from LIBS_CLI.deploy_oSqlScripts import deploy_oSqlScripts
from LIBS_CLI.deploy_oVariables import deploy_oVariables
from LIBS_CLI.deploy_oWorkFlows import deploy_oWorkFlows
from LIBS_CLI.deploy_oWidgets import deploy_oWidgets

def action_3(script_type, object_name, project, source_tenant, target_tenant):
    console = Console()

    #Creating tenants competcs
    tenant_src=cli_create_tenant(source_tenant)
    tenant_trg=cli_create_tenant(target_tenant)

    #Creating API compects
    onyx_src=cli_connect_api(tenant_src)
    onyx_trg=cli_connect_api(tenant_trg)

    #GET target id
    source_id=find_onyx_project(onyx_src, project)
    target_id=find_onyx_project(onyx_trg, project)

    #Creating project for comparaison
    component_list=["CONNECTION","NOTIFICATION","DATAPIPELINE","FORM","SHELL_SCRIPT","SQL_SCRIPT","FILE_PROVIDER","VARIABLE","WORKFLOW","REPORT","WIDGET"]

    if script_type not in component_list:
        console.print("Component ",script_type," does not exist")
        console.print("Please select one component in the following list")
        for element in component_list:
            console.print(element)
    else:

        if script_type == "CONNECTION":
            deploy_oConnections(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "NOTIFICATION":
            deploy_oNotifications(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "DATAPIPELINE":
                deploy_oDataPipelines(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "FORM":
            deploy_oForms(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        """if script_type == "SCHEDULE":
            deploy_oSchedules(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type)"""

        if script_type == "SHELL_SCRIPT":
            deploy_oShellScripts(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "SQL_SCRIPT":
            deploy_oSqlScripts(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "FILE_PROVIDER":
            deploy_oFileProviders(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "VARIABLE":
            deploy_oVariables(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "WORKFLOW":
            deploy_oWorkFlows(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "REPORT":
            deploy_oReports(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)

        if script_type == "WIDGET":
            deploy_oWidgets(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name)








from CLASSES import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects


def deploy_oPipelineColumns(onyx_src : NxOnyxApi, onyx_trg: NxOnyxApi, tenant_src, tenant_trg, source_id, target_id, script_type, object_name,
                         new_datapipeline_id):


    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name), object=script_type, name=object_name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name), object=script_type, name=object_name)
    comp = compare_onyx_projects(project_src, project_tnt)

    lst=[]
    for obj in comp.componants:
        if "oColumnDatapipeline" in obj.content and (obj.action_required == 'U' or obj.action_required == 'C'):
            data = obj.content["oColumnDatapipeline"]
            response = onyx_trg.createPipelineColumn(oFlowId=new_datapipeline_id, sourceColumnName=data["sourceColumnName"], destinationColumnName=data["destinationColumnName"],
                                    destinationColumnDataType=data["destinationColumnDataType"], sourceCharacterMaximumLength=data["sourceCharacterMaximumLength"],
                                    sourceColumnNumericPrecision=data["sourceColumnNumericPrecision"], sourceColumnNumericScale=data["sourceColumnNumericScale"],
                                     sourceColumnDatatype=data["sourceColumnDatatype"], ordinal=data["ordinal"],missingInSource=data["missingInSource"],id=obj.content["trg_id"])





from CLASSES import NxOnyxApi


def cli_get_organisation_unit_name(onyx : NxOnyxApi,organisationunitId):
    for item in onyx.getOrganisationUnits()["items"]:
        if organisationunitId== item["id"]:
            return item["displayName"]

def cli_get_organisation_unit_id(onyx: NxOnyxApi, name):
    unit_id_exists=None
    for item in onyx.getOrganisationUnits()["items"]:
        if name == item["displayName"]:
            unit_id_exists = item["id"]
    if unit_id_exists is None:
        print("Please create your organization unit before executing this command")
    return unit_id_exists




from CLASSES import NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects
from LIBS_CLI.deploy_oConnections import deploy_oConnections
from LIBS_CLI.cli_get_connection import cli_get_connection_datapipeline, cli_get_connection_name
from LIBS_CLI.deploy_oPipelineColumns import deploy_oPipelineColumns

def deploy_oDataPipelines(onyx_src : NxOnyxApi, onyx_trg: NxOnyxApi, tenant_src, tenant_trg, source_id, target_id, script_type, object_name):


    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name), object=script_type, name=object_name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name), object=script_type, name=object_name)
    comp = compare_onyx_projects(project_src, project_tnt)

    response=None

    for obj in comp.componants:
        response = obj.content["trg_id"]
        if "oDatapipeline" in obj.content:

            src_conn_id = cli_get_connection_datapipeline(obj.content["oDatapipeline"]["sourceConnectionId"], target_id, onyx_src, onyx_trg)
            if src_conn_id is None:
                object_name = cli_get_connection_name(obj.content["oDatapipeline"]["sourceConnectionId"], onyx_src)
                src_conn_id = deploy_oConnections(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id,
                                              "CONNECTION", object_name)

            trg_conn_id = cli_get_connection_datapipeline(obj.content["oDatapipeline"]["destinationConnectionId"], target_id,
                                                          onyx_src, onyx_trg)
            if trg_conn_id is None:
                object_name = cli_get_connection_name(obj.content["oDatapipeline"]["destinationConnectionId"], onyx_src)
                trg_conn_id = deploy_oConnections(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id,
                                                  "CONNECTION", object_name)

            response=create_oDatapipeline(obj, target_id, onyx_trg, src_conn_id, trg_conn_id)

            deploy_oPipelineColumns(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, script_type, object_name,response)

    return response


def create_oDatapipeline(obj,target_id, onyx_trg, src_conn_id, trg_conn_id):
    response = obj.content["trg_id"]
    if "oDatapipeline" in obj.content and (obj.action_required == 'U' or obj.action_required == 'C'):

        data = obj.content["oDatapipeline"]
        response = onyx_trg.createPipeline(id=obj.content["trg_id"], oProjectId=target_id, SourceConnectionId=src_conn_id,
                                           SourceConnectionType=data["sourceConnectionType"],
                                           DestinationConnectionType=data["destinationConnectionType"],
                                           DestinationConnectionId=trg_conn_id, Name=data["name"],
                                           SourceTable=data["sourceTable"],
                                           DestinationTable=data["destinationTable"], QueryFilter=data["queryFilter"],
                                           NbColumns=data["nbColumns"],
                                           ActionIfTableExists=data["actionIfTableExists"],
                                           ActionIfTableNotExists=data["actionIfTableNotExists"],
                                           DesactivateIndexes=data["desactivateIndexes"],
                                           CreatePrimaryKey=data["createPrimaryKey"],
                                           CreateIndexes=data["createIndexes"],
                                           NumberOfSchedules=data["numberOfSchedules"])
    return response


from CLASSES.NxOnyxApi import NxOnyxApi

def find_onyx_project(onyx: NxOnyxApi, name):
    # Recuperation de l arbre des projects
    tree = onyx.getTree()
    return search_object_in_tree(tree, name)

def search_object_in_tree(tree, name):
    # Recuperation de l arbre des projects
    id = None
    for object in tree:
        if object["detailType"] == 'project':
            if object["name"] == name:
                id = object["projectId"]
    return id


from rich.console import Console
from rich.markdown import Markdown

def display_env(env):
    console = Console()
    console.print(Markdown("""
> Client: {}
>
> Source env: {}
>
> Target env: {}
---
""".format(env["client"], env["source"], env["target"])))





from .apply_transco import apply_transco
from .compare_onyx_projects import compare_onyx_projects, obj_diff
from .find_onyx_project import find_onyx_project
from .load_env import load_env
from .read_onyx_project_api import read_onyx_project_api
from .read_onyx_project_package import read_onyx_project_package
from .write_to_disk_onyx_project import write_to_disk_onyx_project



from CLASSES import NxProject, NxTenant
from CLASSES.NxOnyxApi import NxOnyxApi
from LIBS_BUS import read_onyx_project_api, compare_onyx_projects
from LIBS_CLI.deploy_oConnections import deploy_oConnections
from LIBS_CLI.cli_get_connection import cli_get_connection_sql, cli_get_connection_name


def deploy_oSqlScripts(onyx_src: NxOnyxApi, onyx_trg: NxOnyxApi, tenant_src: NxTenant,
                      tenant_trg: NxTenant, source_id, target_id, script_type,name):


    project_src = read_onyx_project_api(onyx_src, source_id, tenant_name=(tenant_src.tenant_name),
                                        object=script_type,name=name)
    project_tnt = read_onyx_project_api(onyx_trg, target_id, tenant_name=(tenant_trg.tenant_name),
                                        object=script_type,name=name)
    comp = compare_onyx_projects(project_src, project_tnt)

    response = None
    for obj in comp.componants:
        conn_id=deploy_oConnections(onyx_src, onyx_trg, tenant_src, tenant_trg, source_id, target_id, "CONNECTION",obj.content["osqlScript"]["oConnectionName"])

        response=create_oSqlScripts(obj, target_id, onyx_trg, conn_id)

    return response

def create_oSqlScripts(obj, id, onyx_trg: NxOnyxApi,conn_id):
    id = str(id)
    response = None

    if "osqlScript" in obj.content and obj.action_required == 'C' or obj.action_required == 'U':
        data = obj.content["osqlScript"]
        response=onyx_trg.createSqlScript(id=obj.content["trg_id"], name=data["name"], oConnectionId=conn_id, oProjectId=id,
                                 commandTimeout=data["commandTimeout"], query=data["query"],
                                 documentation=data["documentation"])
    return response





from rich.console import Console
from LIBS_CLI.exist_tenantName import exist_tenantname
from LIBS_CLI.get_envconf import get_envconf
from LIBS_CLI.list_project import list_project


def action_5(tenant):
    console = Console()
    if exist_tenantname(tenantname=tenant) == True:
        i = 1
        for element in list_project(tenant=tenant):
            console.print(str(i) + ' ' + element, style="yellow")
            i += 1
    else:

        console.print("Please configure the tenant parameters before executing this command.", style="red")
